--[[
	MetaGuildNews.lua
		All Functions for displaying and updating the Guild Roster
--]]
local MG = LibStub('AceAddon-3.0'):GetAddon('MetaGuild')
local L = LibStub("AceLocale-3.0"):GetLocale("MetaGuild");
local self = MG

local METAGUILD_NEWS_BUTTON_OFFSET = 2
local METAGUILD_NEWS_BUTTON_HEIGHT = 16
local METAGUILD_NEWS_BUTTON_COUNT = 17

function MetaGuildNewsFrame_OnLoad(widget)

end

function MetaGuildNewsFrame_OnEvent(widget, event, ...)
	if ( not widget:IsShown() ) then
		return
	end
end

function MetaGuildNewsFrame_OnShow(widget)
	MetaGuildNewsFrameEditFilterButton:SetText(L["News/Filter"])
	
	MetaGuildNews_Update()
end

function MetaGuildNewsFrame_OnHide(widget)
	MetaGuildNewsFilterFrame_Show(false)
end


function MetaGuildNewsFilterFrame_Show(force)
	
	if force ~= nil or MetaGuildNewsFiltersFrame:IsShown() then
		MetaGuildNewsFiltersFrame:Hide()
		PlaySound("igMainMenuOptionCheckBoxOff")
	else
		MetaGuildNewsFiltersFrame:Show()
		PlaySound("igMainMenuOptionCheckBoxOn")
	end	
end


function MetaGuildNewsFiltersFrame_OnLoad()
	local filters = {L["News/Filter/MemberAchievements"], L["News/Filter/MemberLevels"], L["News/Filter/GuildMembership"]}
	for i = 1, #filters do
		local checkBox = getglobal("MetaGuildNewsFiltersFrameFilterButton"..i)
		getglobal("MetaGuildNewsFiltersFrameFilterButton"..i.."Text"):SetText(filters[i])

		if ( checkBox ) then
			if ( true ) then
				checkBox:SetChecked(true);
			else
				checkBox:SetChecked(false);
			end
		end
	end

	PlaySound("igMainMenuOptionCheckBoxOn")
end

function MetaGuildNewsFiltersFrame_OnShow()
	MetaGuildNewsFiltersFrameTitle:SetText(L["News/Filter/Title"])
end

function MetaGuildNewsFilter_OnClick(widget)
	if ( widget:GetChecked() ) then
		PlaySound("igMainMenuOptionCheckBoxOn");
	else
		PlaySound("igMainMenuOptionCheckBoxOff");
	end
	--SetGuildNewsFilter(self:GetID(), setting);
	
	--Note Update the list here
	MetaGuildNews_Update()

end

function MetaGuildNewsButton_OnEnter(widget)
	MG:Print("entered")
end

function MetaGuildNewsButton_OnLeave(widget)
	MG:Print("leaving")
end

function MetaGuildNewsButton_OnClick(self, button)
	MG:Print("Click")
end

function MG:GetEventTypeName(typeId)
	if typeId == 1 then
		return "Achievements"
	elseif typeId == 2 then
		return "Levels"
	elseif typeId == 3 then
		return "Memberships"
	end
end

function MG:AddNewsEvent(name, sendTime, eventTypeId, data)
	self:Print("Adding a news event")
	if eventTypeId >= 1 and eventTypeId <= 3 then
		local isNew = true
		local eventType = strlower(self:GetEventTypeName(eventTypeId))
		--self:Print("Adding "..eventType.." event to database")
		
		if self.db.realm.news == nil then
			self.db.realm.news = {["time"]=0, ["achievements"]={}, ["levels"]={}, ["memberships"]={}}
		end		

		if self.db.realm.news[eventType] == nil then
			self.db.realm.news[eventType] = {}
		end
		
		for i,v in ipairs(self.db.realm.news[eventType]) do
			if self.db.realm.news[eventType][i].name == name and self.db.realm.news[eventType][i].time == sendTime and self.db.realm.news[eventType][i].data == data then
				isNew = false
				break
			end
		end
		if isNew then
			self:Print("Saving event to database")
			self.db.realm.news.time = sendTime
			table.insert(self.db.realm.news[eventType], {["name"]=name, ["time"]=sendTime, ["data"]=data})
		end
	end
end

function MetaGuildNewsSort(a,b)
	--local container = getglobal("MetaGuildRosterFrameContainer")
	--local sortType = container.sortType
	local sortType = "time"
	local sortAsc = false
	--self:Print("a: "..type(a[sortType]))
	--self:Print("b: "..type(b[sortType]))
	

	if sortType ~= nil and type(a[sortType]) == type(b[sortType]) then
		if a[sortType] == nil then
			a[sortType] = 0	
		end
		
		if b[sortType] == nil then
			b[sortType] = 0	
		end
		
		if sortAsc then
			return a[sortType]<b[sortType]
		else
			return a[sortType]>b[sortType]
		end
	else
		return true
	end
end




function MetaGuildNews_Update()
	if getglobal("MetaGuildNewsFrame"):IsVisible() then
		local line; -- 1 through 15 of our window to scroll
		local lineplusoffset; -- an index into our data calculated from the scroll offset
		local newsStr
		local i=0
		
		orderedNews = {}
		
		if self.db.realm.news == nil then
			self.db.realm.news = {}
		end
		
		
		--Achievements
		if self.db.realm.news.achievements == nil then
			self.db.realm.news.achievements = {}
		end
		if getglobal("MetaGuildNewsFiltersFrameFilterButton1"):GetChecked() then
			for k, v in ipairs(self.db.realm.news.achievements) do
				local link = self:removeLinks(GetAchievementLink(v.data))
				newsStr = format(L["AchievementEarned"], v.name, self:addLinks(link))
				tinsert(orderedNews, {["text"]=newsStr, ["time"]=v.time, ["type"]=1})
				i = i+1
			end
		end

		--Player Level Ups
		if self.db.realm.news.levels == nil then
			self.db.realm.news.levels = {}
		end
		if getglobal("MetaGuildNewsFiltersFrameFilterButton2"):GetChecked() then
			for k, v in ipairs(self.db.realm.news.levels) do
				newsStr = format(L["LevelEarned"], v.name, v.data)
				tinsert(orderedNews, {["text"]=newsStr, ["time"]=v.time, ["type"]=2})
				i = i+1
			end
		end
		
		--Guild Memberships
		if self.db.realm.news.memberships == nil then
			self.db.realm.news.memberships = {}
		end
		if getglobal("MetaGuildNewsFiltersFrameFilterButton3"):GetChecked() then
			for k, v in ipairs(self.db.realm.news.memberships) do
				if v.data >= 1 then
					newsStr = format(L["PromotionGiven"], v.name, self:GetRankName(v.data))
				elseif v.data == 0 then
					newsStr = format(L["Joined"], v.name)
				elseif v.data < 0 and v.data >= -3 then
					newsStr = format(L["DemotionGiven"], v.name, self:GetRankName(abs(v.data)))
				elseif v.data == -100 then
					newsStr = format(L["Kicked"], v.name)
				end
				tinsert(orderedNews, {["text"]=newsStr, ["time"]=v.time, ["type"]=3})
				i = i+1
			end
		end
		

		--Sorting Start
		sort(orderedNews, MetaGuildNewsSort)
		
		totalNews = table.getn(orderedNews)
		--self:Print("totalNews:"..totalNews)
		FauxScrollFrame_Update(MetaGuildNewsFrameContainer,totalNews,METAGUILD_NEWS_BUTTON_COUNT,METAGUILD_NEWS_BUTTON_HEIGHT)
		--FauxScrollFrame_Update(MetaGuildNewsFrameContainer,totalNews,METAGUILD_NEWS_BUTTON_COUNT,18)

		for line=1,METAGUILD_NEWS_BUTTON_COUNT do
			lineplusoffset = line + FauxScrollFrame_GetOffset(MetaGuildNewsFrameContainer)
			--self:Print("FauxLineOffset: "..FauxScrollFrame_GetOffset(MetaGuildNewsFrameContainer))
			local frame = getglobal("MetaGuildNewsFrameEntry"..line)
			local entry = orderedNews[lineplusoffset]
			--self:Print("LineOffset: "..lineplusoffset)
			if lineplusoffset <= totalNews  and entry ~= nil then
				buttonString = getglobal("MetaGuildNewsFrameEntry"..line)				
				buttonString.text:SetText(entry["text"] )

				getglobal(frame:GetName()):Show();
			else
				getglobal(frame:GetName()):Hide();
			end
		end
	end
end
