--[[
	MetaGuildRoster.lua
		All Functions for displaying and updating the Guild Roster
--]]
local MG = LibStub('AceAddon-3.0'):GetAddon('MetaGuild')
local L = LibStub("AceLocale-3.0"):GetLocale("MetaGuild");
local self = MG


local METAGUILD_DETAIL_NORM_HEIGHT = 195
local METAGUILD_DETAIL_OFFICER_HEIGHT = 255
local METAGUILD_ROSTER_MAX_COLUMNS = 5;
local METAGUILD_ROSTER_MAX_STRINGS = 5;
local METAGUILD_ROSTER_BAR_MAX = 239;
local METAGUILD_ROSTER_BUTTON_OFFSET = 2;
local METAGUILD_ROSTER_BUTTON_HEIGHT = 20;
local METAGUILD_ROSTER_BUTTON_COUNT = 15;
local currentGuildView;
local orderedRoster = {}





-- global for localization changes
local METAGUILD_ROSTER_COLUMNS = {
	playerStatus = {["name"]="Player Status", ["columns"]={"level", "class", "wideName", "zone"}},
	guildStatus = {["name"]="Guild Status", ["columns"]={"name", "rank", "note", "online" }},
--	weeklyxp = {["name"]="Weekly XP", ["columns"]={"level", "class", "wideName", "weeklyxp" }},
--	totalxp = {["name"]="Total XP", ["columns"]={"level", "class", "wideName", "totalxp" }},
	pvpRating = {["name"]="PVP Rating", ["columns"]={"wideName", "ratingBg", "rating2v2", "rating3v3", "rating5v5"}},
	professions = {["name"]="Professions", ["columns"]={"wideName", "zone", "skill"}},
}


local METAGUILD_ROSTER_COLUMN_DATA = {
	level = { width = 32, text = LEVEL_ABBR, stringJustify="CENTER" },
	class = { width = 32, text = CLASS_ABBR, hasIcon = true },
	name = { width = 81, text = NAME, stringJustify="LEFT" },
	wideName = { width = 115, text = NAME, sortType = "name", stringJustify="LEFT" },
	rank = { width = 82, text = RANK, stringJustify="LEFT" },
	note = { width = 76, text = LABEL_NOTE, stringJustify="LEFT" },
	online = { width = 74, text = LASTONLINE, stringJustify="LEFT" },
	zone = { width = 134, text = ZONE, stringJustify="LEFT" },	
	ratingBg = { width = 49, text = "BG", stringJustify="LEFT" },
	rating2v2 = { width = 49, text = "2v2", stringJustify="LEFT" },
	rating3v3 = { width = 49, text = "3v3", stringJustify="LEFT" },
	rating5v5 = { width = 49, text = "5v5", stringJustify="LEFT" },
	weeklyxp = { width = 144, text = GUILD_XP_WEEKLY, stringJustify="RIGHT", hasBar = true },
	totalxp = { width = 144, text = GUILD_XP_TOTAL, stringJustify="RIGHT", hasBar = true },
	achievement = { width = 144, text = ACHIEVEMENT_POINTS, stringJustify="RIGHT", sortType="achievementpoints", hasBar = true },
	skill = { width = 63, text = SKILL_POINTS_ABBR, stringJustify="LEFT" },
}


function MG:AddMainToName(name)
	local nameStr = name
	local main = self:GetMainFromName(name)

	if(name ~= main) then
		return (name.." ("..main..")")
	else
		return name
	end
end

function MetaGuildRosterFrame_OnLoad(frame)
	--self:Print("OnLoad")

	--GuildFrame_RegisterPanel(self);
	--GuildRosterContainer.update = GuildRoster_Update;
	--HybridScrollFrame_CreateButtons(GuildRosterContainer, "GuildRosterButtonTemplate", 0, 0, "TOPLEFT", "TOPLEFT", 0, -GUILD_ROSTER_BUTTON_OFFSET, "TOP", "BOTTOM");
	--GuildRosterContainerScrollBar.doNotHide = true;
	--GuildRosterShowOfflineButton:SetChecked(GetGuildRosterShowOffline());
	--self:RegisterEvent("GUILD_TRADESKILL_UPDATE");
	--self:RegisterEvent("GUILD_ROSTER_UPDATE");
	--self:RegisterEvent("GUILD_RECIPE_KNOWN_BY_MEMBERS");
	--self:RegisterEvent("PLAYER_ENTERING_WORLD");
	--GuildRoster_SetView(GetCVar("guildRosterView"));
	--SetGuildRosterSelection(0);
	--UIDropDownMenu_SetSelectedValue(GuildRosterViewDropdown, currentGuildView);
	--self.doRecipeQuery = true;
	
	local container = getglobal("MetaGuildRosterFrameContainer")
	container.sortAsc= true
	container.sortType= "name"

	--Set the text for the frame
	getglobal("MetaGuildRosterFrameViewDropdownViewLabel"):SetText(L["Roster/ViewLabel"]);
	
	self:Roster_SetView(view)
end

function MetaGuildRosterFrame_OnEvent(self, event, ...)
--	if ( not self:IsShown() ) then
--		return;
--	end
--	if ( event == "GUILD_TRADESKILL_UPDATE" ) then
--		if ( currentGuildView == "tradeskill" ) then
--			GuildRoster_Update();
--		end
--	elseif ( event == "GUILD_ROSTER_UPDATE" ) then
--		if ( currentGuildView ~= "tradeskill" ) then
--			local arg1 = ...;
--			if ( arg1 ) then
--				GuildRoster();
--			end		
--			GuildRoster_Update();
--		end
--	elseif ( event == "PLAYER_ENTERING_WORLD" ) then
--		self.doRecipeQuery = true;
--	end
end


function MetaGuildRoster_Update()
	local RosterFrame = getglobal("MetaGuildRosterFrame")


	if RosterFrame:IsVisible() then
		--self:Print("Updating Roster")
		local line; -- 1 through 15 of our window to scroll
		local lineplusoffset; -- an index into our data calculated from the scroll offset
		local checkBox = getglobal("MetaGuildRosterFrameShowOfflineButton")
		local character
		local i=0
		orderedRoster = {}

		for k, v in pairs(self.db.realm.onlinePlayers) do
			local character = self:GetCharacterInfo(k)
			if character ~= nil then
				character.rank = self:GetRankName(character.rank)
				tinsert(orderedRoster, character)
				i = i+1
			else
				self:Print("Unable to get Character info for "..k)
			end
		end
		
		--self:Print("There are "..i.." characters in the ordered List after proccessing the online players list.")

		if checkBox:GetChecked() then
			for k, v in pairs(self.db.realm.offlinePlayers) do
				local isOffline = true
				for a, b in pairs(v.characters) do
					for y, z in pairs(self.db.realm.onlinePlayers) do
						if y == a then
							isOffline = false
						end
					end
				end
				if isOffline then
					character = self:GetCharacterInfo(k)
					if character ~= nil then
						if character.rank == nil then
							self:Print("Character: "..k)
						end
						character.rank = self:GetRankName(character.rank)
						tinsert(orderedRoster, character)
						i = i+1
					else
						self:Print("Could not load character: "..k)
					end
				end
			end
		end
		
		--self:Print("There are "..i.." characters in the ordered List after proccessing the offline players list.")

		totalPlayers = table.getn(orderedRoster)
		FauxScrollFrame_Update(MetaGuildRosterFrameContainer,totalPlayers,METAGUILD_ROSTER_BUTTON_COUNT,METAGUILD_ROSTER_BUTTON_HEIGHT);

		currentGuildView = UIDropDownMenu_GetSelectedValue(MetaGuildRosterFrameViewDropdown)
		--self:Print("Current View: "..currentGuildView)

		--Sorting Start
		sort(orderedRoster, MetaGuildRosterSort)
		--Sorting end


		for line=1,METAGUILD_ROSTER_BUTTON_COUNT do
			lineplusoffset = line + FauxScrollFrame_GetOffset(MetaGuildRosterFrameContainer);
			local frame = getglobal("MetaGuildRosterFrameEntry"..line)
			local entry = orderedRoster[lineplusoffset]
			--self:Print("LineOffset: "..lineplusoffset)
			if lineplusoffset <= totalPlayers  and entry ~= nil then
				buttonString = getglobal("MetaGuildRosterFrameEntry"..line)
				--GuildRosterButton_SetStringText(buttonString.string1, 48, 1, "Druid")
			--	getglobal(buttonString):SetText(orderedRoster[lineplusoffset-1]["level"]);
			--	getglobal("MetaGuildRosterFrameEntry"..line)["string2"]:SetText(orderedRoster[lineplusoffset-1]["class"]);
				--self:Print("Frame: "..frame:GetName())
				--self:Print("Frame: "..frame:GetName())
				--getglobal(frame:GetName()):SetText(85);

				self:GuildRosterButton_SetStringText(frame.string1, " ", false)
				self:GuildRosterButton_SetStringText(frame.string2, " ", false)
				self:GuildRosterButton_SetStringText(frame.string3, " ", false)
				local nameStr = self:AddMainToName(entry["name"])
				local isOnline = self:CharacterIsOnline(entry["name"])

				if entry["class"] == "???" then
					frame.icon:Hide()
				end
				
				if entry["skill"] == nil then
					entry["skill"] = "???"
				end
				
				if currentGuildView == "playerStatus" then
	--				if entry["class"] == "" then
	--					entry["class"] = "WARRIOR"
	--
	--				end
					if entry["class"] == "???" then
						frame.icon:Hide()
					else
						frame.icon:Show()
						frame.icon:SetTexCoord(unpack(CLASS_ICON_TCOORDS[string.upper(entry["class"])]))
					end
					self:GuildRosterButton_SetStringText(frame.string1, entry["level"], isOnline)
					self:GuildRosterButton_SetStringText(frame.string2, nameStr, isOnline, string.upper(entry["class"]))
					self:GuildRosterButton_SetStringText(frame.string3, entry["zone"], isOnline)
				elseif ( currentGuildView == "guildStatus" ) then
					timeSinceLastOnline = self:GetLastTimeOnline(entry["time"], entry["status"])
					self:GuildRosterButton_SetStringText(frame.string1, entry["name"], isOnline, string.upper(entry["class"]))
					self:GuildRosterButton_SetStringText(frame.string2, entry["rank"], isOnline)
					self:GuildRosterButton_SetStringText(frame.string3, entry["public"], isOnline)
					self:GuildRosterButton_SetStringText(frame.string4, timeSinceLastOnline, isOnline)
				elseif ( currentGuildView == "pvpRating" ) then
					self:GuildRosterButton_SetStringText(frame.string1, entry["name"], isOnline, string.upper(entry["class"]))
					self:GuildRosterButton_SetStringText(frame.string2, entry["ratingBg"], isOnline)
					self:GuildRosterButton_SetStringText(frame.string3, entry["rating2v2"], isOnline)
					self:GuildRosterButton_SetStringText(frame.string4, entry["rating3v3"], isOnline)
					self:GuildRosterButton_SetStringText(frame.string5, entry["rating5v5"], isOnline)
					--self:GuildRosterButton_SetStringText(frame.string3, (entry.pvp["rating2v2"].." / "..entry.pvp["rating3v3"].." / "..entry.pvp["rating5v5"]), isOnline)
				elseif ( currentGuildView == "professions" ) then
					self:GuildRosterButton_SetStringText(frame.string1, entry["name"], isOnline, string.upper(entry["class"]))
					self:GuildRosterButton_SetStringText(frame.string2, entry["zone"], isOnline)
					self:GuildRosterButton_SetStringText(frame.string3, entry["skill"], isOnline)
				
				end
				frame.name = entry["name"]
				frame.main = entry["main"]
				--MetaGuildRosterFrameEntry1.string1:SetText(entry["level"])

				getglobal(frame:GetName()):Show();
			else
				getglobal(frame:GetName()):Hide();
			end
		end
		--self:Print("We're at "..FauxScrollFrame_GetOffset(MetaGuildRosterFrameContainer));
		
		
		--update the info frame
		local infoFrame = getglobal("MetaGuild_PlayerInfoFrame")
		if infoFrame.name ~= nil then
			MetaGuild_PlayerinfoUpdate(infoFrame.name)
		end
	end
end


function MetaGuildRosterFrame_OnShow(self)
--	MetaGuildRoster_RecipeQueryCheck();
	MetaGuildRoster_Update()
end

function MetaGuildRoster_RecipeQueryCheck()
--	if ( GuildRosterFrame.doRecipeQuery ) then
--		QueryGuildRecipes();
--		GuildRosterFrame.doRecipeQuery = nil;
--	end
end

function MetaGuildRoster_GetLastOnline(guildIndex)
	--return RecentTimeDate( GetGuildRosterLastOnline(guildIndex) );
end

function MetaGuildRoster_SortByColumn(column)
	--self:Print("Sorting Column "..column.sortType)
	if ( column.sortType ) then
		local container = getglobal("MetaGuildRosterFrameContainer")
		container.sortType = column.sortType
		if container.sortAsc then
			container.sortAsc = false
		else
			container.sortAsc = true
		end
		--self:Print("SortDir: "..container.sortDir)
		MetaGuildRoster_Update()
	end
	PlaySound("igMainMenuOptionCheckBoxOn")
end


function MetaGuildRosterClickHandler(widget, button)
	if (button == "LeftButton") then
		MetaGuild_PlayerinfoShow(widget.name, false)
	elseif (button == "RightButton") then
		FriendsFrame_ShowDropdown(widget.name,true)
	end
	MetaGuildRoster_Update()
end


--****** Guild members **********************************************************

function MG:GuildRosterButton_SetStringText(buttonString, text, isOnline, class)
	buttonString:SetText(text);
	if ( isOnline ) then
		if ( class ) then
			local classColor = RAID_CLASS_COLORS[class]
			buttonString:SetTextColor(classColor.r, classColor.g, classColor.b)
		else
			buttonString:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		end
	else
		buttonString:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
	end
end

function MetaGuild_TabHeader(self, size, text)
    self:SetText(L[text]);
    WhoFrameColumn_SetWidth(self, size);
end


function MetaGuild_PlayerinfoUpdate(name)
	local infoFrame = getglobal("MetaGuild_PlayerInfoFrame")
	local RosterFrame = getglobal("MetaGuildRosterFrame")

	infoFrame.name = name
	infoFrame.main = self:GetMainFromName(name)
	
	local character = self:GetCharacterInfo(name)
		
	if character ~= nil then
		
--		local timeSinceLastOnline = self:TimeSinceTimeCode(character.time)
--		timeSinceLastOnline = self:displayTimeSinceLastOn(timeSinceLastOnline, character.status)
		
		
		timeSinceLastOnline = self:GetLastTimeOnline(character.time, character.status)
		--Note: Change out the labels for localizations...

		getglobal(infoFrame:GetName().."Name"):SetText(self:AddMainToName(character.name))
		getglobal(infoFrame:GetName().."Level"):SetText(L["PlayerInfoFrame/Level"].." "..character.level.." "..L[character.class])
		getglobal(infoFrame:GetName().."ZoneLabel"):SetText(L["PlayerInfoFrame/Zone:"])
		getglobal(infoFrame:GetName().."ZoneText"):SetText(character.zone)
		getglobal(infoFrame:GetName().."RankLabel"):SetText(L["PlayerInfoFrame/Rank:"])
		getglobal(infoFrame:GetName().."RankText"):SetText(self:GetRankName(character.rank))
		getglobal(infoFrame:GetName().."OnlineLabel"):SetText(L["PlayerInfoFrame/LastOnline:"])
		getglobal(infoFrame:GetName().."OnlineText"):SetText(timeSinceLastOnline)

		getglobal(infoFrame:GetName().."_NoteBackground_PersonalNoteText"):SetText(character.public.text)
		getglobal(infoFrame:GetName().."_OfficerNoteBackground_OfficerNoteText"):SetText(character.officer.text)

		--infoFrameTradeSkillLabel:SetText(L["OnlineList/Professions"]);
		--infoFrameTradeSkills:SetText("");
		--self:GetTradeskills(name);
		
		--adjust the window for officers / normal members
		if self:CanViewOfficerNote() then
			getglobal(infoFrame:GetName().."OfficerNoteLabel"):Show()
			getglobal(infoFrame:GetName().."_OfficerNoteBackground"):Show()
			infoFrame:SetHeight(METAGUILD_DETAIL_OFFICER_HEIGHT)
		else
			getglobal(infoFrame:GetName().."OfficerNoteLabel"):Hide()
			getglobal(infoFrame:GetName().."_OfficerNoteBackground"):Hide()
			infoFrame:SetHeight(METAGUILD_DETAIL_NORM_HEIGHT)
		end
		
		--Enable / Disable the guild kick button.
		if self:CanGuildRemove() and self:IsPlayersCharacter(name) ~= true then
			getglobal(infoFrame:GetName().."_GuildMemberRemoveButton"):Enable()
		else
			getglobal(infoFrame:GetName().."_GuildMemberRemoveButton"):Disable();
		end
		
		--Enable / Disable the group invite button.
		if self:IsPlayersCharacter(name) ~= true and self:CharacterIsOnline(name) then
			getglobal(infoFrame:GetName().."_GroupInviteButton"):Enable()
		else
			getglobal(infoFrame:GetName().."_GroupInviteButton"):Disable();
		end
		
		if self:CanPromote(name) then
			getglobal(infoFrame:GetName().."_PromoteButton"):Show()
		else
			getglobal(infoFrame:GetName().."_PromoteButton"):Hide()
			--getglobal(infoFrame:GetName().."_PromoteButton"):Show()
		end
		
		if self:CanDemote(name) then
			getglobal(infoFrame:GetName().."_DemoteButton"):Show()
		else
			getglobal(infoFrame:GetName().."_DemoteButton"):Hide()
			--getglobal(infoFrame:GetName().."_DemoteButton"):Show()
		end	
	end
end


function MG:CanGuildRemove()
	local name = UnitName("player")
	local rank = self:GetRank(name)
	if rank >= 2 then -- Rank 2 is Officer and Rank 3 is Guild Master
		return true
	else
		return false
	end
end

function MetaGuild_PlayerinfoShow(name, forceClose)
	local infoFrame = getglobal("MetaGuild_PlayerInfoFrame")
	local RosterFrame = getglobal("MetaGuildRosterFrame")
	if ((infoFrame:IsShown() and name == infoFrame.name) or forceClose) then
		infoFrame:Hide();
		RosterFrame.selectedGuildMember = nil
	else
		local main = self:GetMainFromName(name)
		RosterFrame.selectedGuildMember = {["name"]=name, ["main"]=main}
		MetaGuild_PlayerinfoUpdate(name)
		infoFrame:Show()
	end
end

function MetaGuildRosterSort(a,b)
	local container = getglobal("MetaGuildRosterFrameContainer")
	local sortType = container.sortType

	--self:Print("a: "..type(a[sortType]))
	--self:Print("b: "..type(b[sortType]))
	

	if sortType ~= nil and type(a[sortType]) == type(b[sortType]) then
		if a[sortType] == nil then
			a[sortType] = 0	
		end
		
		if b[sortType] == nil then
			b[sortType] = 0	
		end
		
		if container.sortAsc then
			return a[sortType]<b[sortType]
		else
			return a[sortType]>b[sortType]
		end
	else
		return true
	end
end

function MetaGuildRoster_UpdateTradeSkills()
--	local scrollFrame = GuildRosterContainer;
--	local offset = HybridScrollFrame_GetOffset(scrollFrame);
--	local buttons = scrollFrame.buttons;
--	local numButtons = #buttons;
--	local button, index, class;
--	local numTradeSkill = GetNumGuildTradeSkill();
	
--	for i = 1, numButtons do
--		button = buttons[i];
--		index = offset + i;
--		if ( index <= numTradeSkill ) then
--			button.guildIndex = index;
--			local skillID, isCollapsed, iconTexture, headerName, numOnline, numPlayers, playerName, class, online, zone, skill, classFileName = GetGuildTradeSkillInfo(index);
--			button.online = online;
--			if ( headerName ) then
--				GuildRosterButton_SetStringText(button.string1, headerName, 1);
--				GuildRosterButton_SetStringText(button.string2, "", 1);
--				GuildRosterButton_SetStringText(button.string3, numOnline, 1);
--				button.header:Show();
--				button:UnlockHighlight();
--				button.header.icon:SetTexture(iconTexture);
--				button.header.name:SetText(headerName);
--				button.header.collapsed = isCollapsed;
--				if ( numPlayers == 0 ) then
--					button.header:Disable();
--					button.header.icon:SetDesaturated(true);
--					button.header.collapsedIcon:Hide();
--					button.header.expandedIcon:Hide();
--					button.header.allRecipes:Hide();
--					button.header.name:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
--					button.header.leftEdge:SetVertexColor(0.75, 0.75, 0.75);
--					button.header.rightEdge:SetVertexColor(0.75, 0.75, 0.75);
--					button.header.middle:SetVertexColor(0.75, 0.75, 0.75);
--				else
--					button.header:Enable();
--					button.header.icon:SetDesaturated(false);
--					if ( CanViewGuildRecipes(skillID) ) then
--						button.header.allRecipes:Show();
--					else
--						button.header.allRecipes:Hide();
--					end
--					button.header.name:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
--					button.header.leftEdge:SetVertexColor(1, 1, 1);
--					button.header.rightEdge:SetVertexColor(1, 1, 1);
--					button.header.middle:SetVertexColor(1, 1, 1);
--					if ( isCollapsed ) then
--						button.header.collapsedIcon:Show();
--						button.header.expandedIcon:Hide();
--					else
--						button.header.expandedIcon:Show();
--						button.header.collapsedIcon:Hide();
--					end
--				end
--				button.header.skillID = skillID;
--			else
--				GuildRosterButton_SetStringText(button.string1, playerName, online, classFileName);
--				GuildRosterButton_SetStringText(button.string2, zone, online);
--				GuildRosterButton_SetStringText(button.string3, "["..skill.."]", online);
--				button.header:Hide();
--			end
--			button:Show();
--			if ( mod(index, 2) == 0 ) then
--				button.stripe:SetTexCoord(0.36230469, 0.38183594, 0.95898438, 0.99804688);
--			else
--				button.stripe:SetTexCoord(0.51660156, 0.53613281, 0.88281250, 0.92187500);
--			end
--		else
--			button:Hide();
--		end
--	end
	
--	local totalHeight = numTradeSkill * (GUILD_ROSTER_BUTTON_HEIGHT + GUILD_ROSTER_BUTTON_OFFSET);
--	local displayedHeight = numButtons * (GUILD_ROSTER_BUTTON_HEIGHT + GUILD_ROSTER_BUTTON_OFFSET);
--	HybridScrollFrame_Update(scrollFrame, totalHeight, displayedHeight);
end

function MetaGuildRosterTradeSkillHeader_OnClick(self)
--	if ( self.collapsed ) then
--		ExpandGuildTradeSkillHeader(self.skillID);
--	else
--		CollapseGuildTradeSkillHeader(self.skillID);
--	end
end

--****** Dropdown ***************************************************************
function MG:Roster_SetView(view)
	--print("Setting View")
	if ( not view or not METAGUILD_ROSTER_COLUMNS[view] ) then
		view = "playerStatus";
	end

	local numColumns = #METAGUILD_ROSTER_COLUMNS[view];
	local stringsInfo = {};
	local stringOffset = 0;
	local haveIcon, haveBar;
	
	-- set up columns
	for columnIndex = 1, METAGUILD_ROSTER_MAX_COLUMNS do
		--local columnButton = _G["MetaGuildRosterFrameColumnButton"..columnIndex];
		--self:Print("ColumnIndex: "..columnIndex)
		
		local columnButton = getglobal("MetaGuildRosterFrameColumnButton"..columnIndex)
		--self:Print("ColumnButton: "..columnButton:GetName())
		local columnType = METAGUILD_ROSTER_COLUMNS[view]["columns"][columnIndex]
		if ( columnType ) then
			local columnData = METAGUILD_ROSTER_COLUMN_DATA[columnType];
			--self:Print("ColumnType: "..columnType)
			columnButton:SetText(columnData.text);
			WhoFrameColumn_SetWidth(columnButton, columnData.width);
			columnButton:Show();
			-- by default the sort type should be the same as the column type
			if ( columnData.sortType ) then
				columnButton.sortType = columnData.sortType;
			else
				columnButton.sortType = columnType;
			end
			
			if ( columnData.hasIcon ) then
				haveIcon = true;
			else	
				-- store string data for processing
				columnData["stringOffset"] = stringOffset;
				table.insert(stringsInfo, columnData);
			end
			--self:Print("Offset: "..stringOffset.." + "..columnData.width.." - 2")
			stringOffset = stringOffset + columnData.width + 2;
			
			haveBar = haveBar or columnData.hasBar;
		else
			columnButton:Hide();
		end
	end	
	
	-- process the button strings
	local button, fontString;
	for buttonIndex = 1, METAGUILD_ROSTER_BUTTON_COUNT do
		button = getglobal("MetaGuildRosterFrameEntry"..buttonIndex)
		for stringIndex = 1, METAGUILD_ROSTER_MAX_STRINGS do
			fontString = button["string"..stringIndex];
--			self:Print("FontString: "..fontString:GetName())
			local stringData = stringsInfo[stringIndex];
			if ( stringData ) then
				-- want strings a little inside the columns, 6 pixels from the left and 8 from the right
				fontString:SetPoint("LEFT", stringData.stringOffset + 6, 0);
				fontString:SetWidth(stringData.width - 14);
				fontString:SetJustifyH(stringData.stringJustify);
				fontString:Show();
			else
				fontString:Hide();
			end
		end
		if ( haveIcon ) then
			button.icon:Show();
		else
			button.icon:Hide();
		end
		if ( haveBar ) then
			button.barLabel:Show()
			-- button.barTexture:Show(); -- shown status determined in GuildRoster_Update 
		else
			button.barLabel:Hide();
			button.barTexture:Hide();
		end
		button.header:Hide();
	end
--	
--	if ( view == "tradeskill" ) then
--		GuildRoster_RecipeQueryCheck();
--	end
--	currentGuildView = view;
end

function MetaGuildRosterViewDropdown_OnLoad(widget)
	local defaultView = "playerStatus"
	UIDropDownMenu_Initialize(widget, MetaGuildRosterViewDropdown_Initialize)
	UIDropDownMenu_SetSelectedValue(MetaGuildRosterFrameViewDropdown, defaultView);
end

function MetaGuildRosterViewDropdown_Initialize(widget, level)	
	local info = UIDropDownMenu_CreateInfo()
	info.func = RosterViewDropdown_OnClick
	
	for k,v in pairs(METAGUILD_ROSTER_COLUMNS) do
		info.text = v.name
		info.value = k
		UIDropDownMenu_AddButton(info, level)
	end
end


function RosterViewDropdown_OnClick(widget)
	MG:Roster_SetView(widget.value);
	UIDropDownMenu_SetSelectedValue(MetaGuildRosterFrameViewDropdown, widget.value)
	MetaGuildRoster_Update()
end

--****** Promote/Demote *********************************************************
function MG:CanPromote(targetName)
	local name = UnitName("player")
	local rank = self:GetRank(name)
	local targetRank = self:GetRank(targetName)


	--This will only allow Officers & Guild Masters to promote.
	if rank > targetRank+1 then -- Rank 2 is Officer and Rank 3 is Guild Master
		--self:Print(rank.." can promote "..targetRank.." by 1 level")
		return true
	else
		--self:Print(rank.." cannot promote "..targetRank.." by 1 level")
		return false
	end
end

function MG:CanDemote(targetName)
	local name = UnitName("player")
	local rank = self:GetRank(name)
	local targetRank = self:GetRank(targetName)
	
	--This will only allow Officers & Guild Masters to promote.
	if rank > targetRank and targetRank ~= 0 then -- Rank 2 is Officer and Rank 3 is Guild Master
		return true
	else
		return false
	end
end

function MG:ConfirmGuildPromote(playerName)
	StaticPopupDialogs["ACCEPT_METAGUILD_PROMOTE"] = {
		text =  L["Info/AcceptGuildPromoteText"],
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			self:SendPromote(playerName)
		end,
		sound = "levelup2",
		timeout = 30,
		whileDead = true,
		hideOnEscape = true,
	}
	StaticPopup_Show ("ACCEPT_METAGUILD_PROMOTE", self:AddMainToName(playerName))
end

function MG:ConfirmGuildDemote(playerName)

	StaticPopupDialogs["ACCEPT_METAGUILD_DEMOTE"] = {
		text =  L["Info/AcceptGuildDemoteText"],
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			self:SendDemote(playerName)
		end,
		sound = "levelup2",
		timeout = 30,
		whileDead = true,
		hideOnEscape = true,
	}

	StaticPopup_Show ("ACCEPT_METAGUILD_DEMOTE", self:AddMainToName(playerName))
end

function MetaGuildFramePromoteButton_OnClick(widget, button)
	local name = widget:GetParent().name
	self:Print("Promoting "..name)
	if self:CanPromote(name) then
		self:ConfirmGuildPromote(name)
	end
	PlaySound("UChatScrollButton")
end

function MetaGuildFrameDemoteButton_OnClick(widget, button)
	local name = widget:GetParent().name
	self:Print("Demoting "..name)
	if self:CanDemote(name) then
		self:ConfirmGuildDemote(name)
	end
	PlaySound("UChatScrollButton")
end



--****** Public/Officer Notes *********************************************************
function MG:CanViewOfficerNote()
	local name = UnitName("player")
	local rank = self:GetRank(name)
	if rank >= 2 then -- Rank 2 is Officer and Rank 3 is Guild Master
		return true
	else
		return false
	end
end


function MetaGuildSetNote(noteType)
	if noteType ~= "PUBLIC" and noteType ~= "OFFICER" then
		MG:Print("Invalid Note Type: "..noteType)
		return
	end
	local authorName = UnitName("player")
	local authorMain = MG:GetMainFromName(authorName)
	local authorRank = MG:GetRank(authorName)
	local targetName, targetMain = GetMetaGuildRosterSelection()
	
	local allow = false
	local label
	if noteType == "PUBLIC" and authorRank >= 2 or (authorRank >= 1 and targetMain == authorMain) then
		label = SET_GUILDPLAYERNOTE_LABEL
		allow = true
	elseif noteType == "OFFICER" and authorRank >= 2 then
		label = SET_GUILDOFFICERNOTE_LABEL
		allow = true
	end
	
	if allow then

		
	
		StaticPopupDialogs["SET_METAGUILDNOTE"] = {
			text = label,
			button1 = L["Accept"],
			button2 = L["Cancel"],
			hasEditBox = 1,
			maxLetters = 31,
			editBoxWidth = 260,
			OnAccept = function(self)
				local name, main = GetMetaGuildRosterSelection()
				--MetaGuildRosterSetOfficerNote(name, MG:GetMainFromName(UnitName("player")), self.editBox:GetText())
				MG:SendNote(noteType, authorMain, targetName, self.editBox:GetText())
				--MetaGuild_PlayerinfoUpdate(name, main)
			end,
			OnShow = function(self)
				--local name, main = GetMetaGuildRosterSelection()
				if targetName ~= nil and targetName ~= nil then
					local character = MetaGuildGetCharacterInfo(targetName)
					local noteText
					if noteType == "PUBLIC" then
						noteText = character.public.text
					elseif noteType == "OFFICER" then
						noteText = character.officer.text
					end
					if noteText ~= nil then
						self.editBox:SetText(noteText)
					end
					self.editBox:SetFocus()
				else
					Print("rosterSelection is Nil for some reason... This should not really happen.")
				end
			end,
			OnHide = function(self)
				ChatEdit_FocusActiveWindow();
				self.editBox:SetText("");
			end,
			EditBoxOnEnterPressed = function(self)
				local parent = self:GetParent();
				--local name, main = GetMetaGuildRosterSelection()
				--MetaGuildRosterSetOfficerNote(name, MG:GetMainFromName(UnitName("player")), self.editBox:GetText())
				MG:SendNote(noteType, authorMain, targetName, parent.editBox:GetText())
				--MetaGuild_PlayerinfoUpdate(name, main)
				parent:Hide();
			end,
			EditBoxOnEscapePressed = function(self)
				self:GetParent():Hide();
			end,
			timeout = 0,
			exclusive = 1,
			whileDead = 1,
			hideOnEscape = 1
		}

		StaticPopup_Show("SET_METAGUILDNOTE")
	end
end