local MG = LibStub("AceAddon-3.0"):NewAddon("MetaGuild", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceComm-3.0", "AceTimer-3.0", "AceSerializer-3.0");
local L = LibStub("AceLocale-3.0"):GetLocale("MetaGuild");
local AceConfigDialog = LibStub("AceConfigDialog-3.0");
local icon = LibStub("LibDBIcon-1.0");
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local LibC = LibStub:GetLibrary("LibCompress")
local LibCE = LibC:GetAddonEncodeTable()

local self = MG;
local specialChar = '\28';
local colorChar = '{%s}';
local specialLinkChar = '\29';
local playerLink = "|Hplayer %s|h[%s]|h";

local status = 1;

-- Online list
local numOnline = 0;

local version = "0.7.2"

local displayNewVersion = true
local doingSync = false

--Right Click Functions
local rightClickFunctions = {}


-- Key Binding Setup
_G["BINDING_HEADER_METAGUILD"] = "Meta Guild";
_G["BINDING_NAME_MG_SHOW"] = "Show player list.";

-- Color Test
local red = "|cffff0000";
local green = "|cff00ff00";
local blue = "|cff0000ff";


local defaults = {
	realm = {
		version = "0.0.0",
		main = nil,
		joined = nil,
		rank = 0,
		timeOffset = 0,
		motd = {["time"]=0},
		mission = {["time"]=0},
		info = {["time"]=0},
		news = {},
		tradeskillPerChar = true,
		professions = {},
		guildName = "",
		guildChannel = "",  --16 char Max
		guildPassword = "",
		baseChannel = "METAGUILD",
		offlinePlayers = {},
		onlinePlayers = {},
		kickedPlayers = {},
		busyPlayers = {},
		updatedPlayers = {},
		syncDelay = 5,
	},
	char = {
		professions = {},
		note = "",
	},
	profile = {
		enabled = true,
		autoSync = true,
		logEnabled = false,
		filterDisabled = false,
		showAchiev = true,
		showLevel = true,
		chatTag = "MetaGuild",
		scale = 1.0,
		color = {
			r = 1.0,
			g = 1.0,
			b = 1.0,
		},
		
	}
}

local dataobj = ldb:NewDataObject("MetaGuild", {
    type = "data source",
	text = "MG (" .. numOnline .. ")",
	label = "MetaGuild",
    icon = "Interface\\AddOns\\MetaGuild\\MinimapIcon.tga",
    OnClick = function(clickedframe, button)
		if button == "LeftButton" then
			MetaGuild_Toggle();
		elseif button == "RightButton" then
			if LibStub("AceConfigDialog-3.0").OpenFrames["MetaGuild"] then
				LibStub("AceConfigDialog-3.0"):Close("MetaGuild")
			else
				LibStub("AceConfigDialog-3.0"):Open("MetaGuild")
			end
		end		 
    end,
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
			if self.db.realm.guildName ~= nil then
				tooltip:AddLine(self.db.realm.guildName)
			else
				tooltip:AddLine("MetaGuild")
			end
			tooltip:AddLine("Members Online:");
			for k, v in pairs(self.db.realm.onlinePlayers) do
				local character = self:GetCharacterInfo(k)
				if character ~= nil then 
					local colors = RAID_CLASS_COLORS[character.class];
					if(colors == nil) then
						colors = {};
						colors.r = 0.5;
						colors.g = 0.5;
						colors.b = 0.5;
					end
					
					tooltip:AddLine("  " .. self:AddMainToName(character.name), colors.r, colors.g, colors.b);
				end
			end
			if (numOnline == 1) then
				tooltip:AddLine(numOnline .. " player");
			else
				tooltip:AddLine(numOnline .. " players");
			end
			tooltip:AddLine("Left-click to open player list", 0, 1, 0)
			tooltip:AddLine("Right-click for options menu", 0, 1, 0)
	end,
})

local options = {
	type = "group",
	name = "MetaGuild Config",
	args = {

		system = {
			type = "group",
			name = "System",
			desc = "System settings",
			order = 50,
			args = {
				enable = {
					name = L["ConfigFrame/Enable"],
					desc = L["ConfigFrame/EnableDesc"],
					type = "toggle",
					order = 55,
					width = "full",
					set = function(info,val) 
						self.db.profile.enabled = val;
						if (val == true) then 
							self:OnEnable();
							self.joinTimer = self:ScheduleRepeatingTimer("JoinMetaGuildChannels", 2);
						else 
							self:OnDisable() 
						end 
					end,
					get = function(info) return self.db.profile.enabled; end
				},			
				main = {
					name = L["ConfigFrame/MainChar"],
					desc = L["ConfigFrame/MainCharDesc"],
					type = "execute",
					order = -2,
					func = function(info) 
						self.db.realm.main = UnitName("player");
						self:SystemMessage(L["MainChanged"]);
						self:SendUpdate();
					end,
				},
				tradeskillPerChar = {
					name = L["ConfigFrame/TradeskillPerChar"],
					desc = L["ConfigFrame/TradeskillPerCharDesc"],
					type = "toggle",
					width = "double",
					set = function(info, val) self.db.realm.tradeskillPerChar = val; end,
					get = function(info) return self.db.realm.tradeskillPerChar end,
				},
				showIcon = {
					name = L["ConfigFrame/ShowIcon"],
					desc = L["ConfigFrame/ShowIconDesc"],
					type = "toggle",
					set = function(info, val) MetaGuildDB.hide = not val; if(val) then icon:Show("MetaGuild"); else icon:Hide("MetaGuild"); end end,
					get = function(info) return not MetaGuildDB.hide; end,
				},
				showAchiev = {
					name = L["ConfigFrame/ShowAchiev"],
					desc = L["ConfigFrame/ShowAchievDesc"],
					type = "toggle",
					order = -3,
					set = function(info, val) self.db.profile.showAchiev = val; end,
					get = function(info) return self.db.profile.showAchiev end,
				},
				showLevel = {
					name = L["ConfigFrame/ShowLevel"],
					desc = L["ConfigFrame/ShowLevelDesc"],
					type = "toggle",
					order = -3,
					set = function(info, val) self.db.profile.showLevel = val; end,
					get = function(info) return self.db.profile.showLevel end,
				},
				scale = {
					name = L["ConfigFrame/Scale"],
					desc = L["ConfigFrame/ScaleDesc"],
					type = "range",
					order = 60,
					min = 0.1,
					max = 2,
					step = 0.01,
					isPercent = true,
					set = function(info, val) self.db.profile.scale = val; _G["MetaGuildFrame"]:SetScale(val);  end,
					get = function(info) return self.db.profile.scale end,
				},
				chatColor = {
					name = L["ConfigFrame/ChatColor"],
					desc = L["ConfigFrame/ChatColorDesc"],
					type = "color",
					order = 70,
					set = function(info, r, g, b, a)
						local channel = GetChannelName(self.db.realm.guildChannel);
						if(channel > 0) then
							ChangeChatColor("CHANNEL" .. GetChannelName(self.db.realm.guildChannel), r, g, b); 
						end
						self.db.profile.color.r = r; 
						self.db.profile.color.g = g; 
						self.db.profile.color.b = b;
						--self:SystemMessage(L["ChatColorChanged"]);	-- Spamms the chat
					end,
					get = function(info) return self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b; end,
				},
			},
		},
		debug = {
			type = "group",
			name = "Debug",
			desc = "Guild Debug Controls",
			args = {
				disableFilter = {
					name = L["ConfigFrame/FilterDisabled"],
					desc = L["ConfigFrame/FilterDisabledDesc"],
					type = "toggle",
					order = 55,
					width = "full",
					set = function(info,val) 
						self.db.profile.filterDisabled = val;
					end,
					get = function(info) return self.db.profile.filterDisabled; end
				},
				join = {
					name = L["ConfigFrame/SendJoin"],
					desc = L["ConfigFrame/SendJoinDesc"],
					type = "execute",
					order = -2,
					func = function(info) 
						self:SendJoin();
					end,
				},
				characterUpdate = {
					name = L["ConfigFrame/SendUpdate"],
					desc = L["ConfigFrame/SendUpdateDesc"],
					type = "execute",
					order = -2,
					func = function(info) 
						self:SendUpdate();
					end,
				},
				compData = {
					name = "Data Test",
					desc = "More Tests",
					type = "execute",
					order = 0,
					func = function(info)
						self:DataTest()
					end,
				},
				PvPUpdate = {
					name = "PvP Update",
					desc = "Updates the PvP ratings for the player",
					type = "execute",
					order = 0,
					func = function(info)
						self:SendPvpUpdate(1)
					end,
				},
				wipe = {
					name = L["ConfigFrame/Wipe"],
					desc = L["ConfigFrame/WipeDesc"],
					type = "execute",
					order = 1,
					func = function(info) 
						self.db.realm.offlinePlayers = {}
						self.db.realm.onlinePlayers = {}
						self.db.realm.news = {}
						self.db.realm.motd = {}
						self.db.realm.mission = {}
						self.db.realm.info = {}
						self:SendJoin()
					end,
				},
			},
		},
		log = {
			type = "group",
			name = "Log",
			desc = "Guild Log Controls",
			args = {
				enable = {
					name = L["ConfigFrame/EnableLog"],
					desc = L["ConfigFrame/EnableLogDesc"],
					type = "toggle",
					order = 55,
					width = "full",
					set = function(info,val) 
						self.db.profile.logEnabled = val;
						if (val == true) then
							--MetaGuildFrameInfoTabFrameViewLogButton:Enable()
							MetaGuildInfoFrameViewLogButton:Enable()
							--self:OnEnable();
							--self.joinTimer = self:ScheduleRepeatingTimer("JoinMetaGuildChannels", 2);
						else 
							--MetaGuildFrameInfoTabFrameViewLogButton:Disable()
							MetaGuildInfoFrameViewLogButton:Disable()
							--self:OnDisable() 
						end 
					end,
					get = function(info) return self.db.profile.logEnabled; end
				},
			},
		},
		dev = {
			type = "group",
			name = "Dev",
			desc = "Only Visible to the Developer",
			args = {
				guildName = {
					name = L["ConfigFrame/GuildName"],
					desc = L["ConfigFrame/GuildNameDesc"],
					order = 45,
					width = "full",
					type = "input",
					set = function(info,val)
							local wasEnabled = self.db.profile.enabled
							if type(self.db.realm.guildName) ~= "table" then
								self.db.realm.guildName = {}
							end
							if(wasEnabled) then
								self.db.profile.enabled = false
								self:OnDisable();
							end
							self.db.realm.guildName = val
							if(wasEnabled) then
								self.db.profile.enabled = true
								self:OnEnable()
							end							
						  end,
					get = function(info) return self.db.realm.guildName; end
				},
				guildChannel = {
					name = L["ConfigFrame/GuildChannel"],
					desc = L["ConfigFrame/GuildChannelDesc"],
					order = 46,
					width = "full",
					type = "input",
					set = function(info,val)
							local wasEnabled = self.db.profile.enabled
							if(wasEnabled) then
								self.db.profile.enabled = false
								self:OnDisable()
							end
							self.db.realm.guildChannel = val
							if(wasEnabled) then
								self.db.profile.enabled = true
								self:OnEnable()
							end							
						  end,
					get = function(info) return self.db.realm.guildChannel; end
				},
				guildPassword = {
					name = L["ConfigFrame/GuildPassword"],
					desc = L["ConfigFrame/GuildPasswordDesc"],
					order = 47,
					width = "double",
					type = "input",
					set = function(info,val)
							local wasEnabled = self.db.profile.enabled;
							if(wasEnabled) then
								self.db.profile.enabled = false;
								self:OnDisable();
							end
							self.db.realm.guildPassword = val;
							if(wasEnabled) then
								self.db.profile.enabled = true;
								self:OnEnable();
							end							
						  end,
					get = function(info) return self.db.realm.guildPassword; end
				},
				rank = {
					name = L["ConfigFrame/Rank"],
					desc = L["ConfigFrame/RankDesc"],
					type = "range",
					order = 60,
					min = 0,
					max = 3,
					step = 1.0,
					isPercent = false,
					set = function(info, val) self.db.realm.rank = val; self:SendUpdate(); MetaGuildGuild_Update(); end,
					get = function(info) return self.db.realm.rank end,
				},
				autoSync = {
					name = "AutoSync",
					desc = "This will turn off the Auto Sync everytime you log in",
					type = "toggle",
					order = 55,
					width = "full",
					set = function(info,val) 
						self.db.profile.autoSync = val
					end,
					get = function(info) return self.db.profile.autoSync end
				},
			},
		},
		show = {
			name = L["ConfigFrame/ShowPlayerlist"],
			desc = L["ConfigFrame/ShowPlayerlistDesc"],
			guiHidden  = true,
			type = "execute",
			func = function(info) MetaGuild_OnlineFrame:Show(); end,
		},
	},
	
}

function MG:CreateMsgTemplate(dataFieldCount)
	return string.rep(specialChar .. "%s", dataFieldCount) .. specialChar;
end

function MG:OnInitialize()
	-- do init tasks here, like loading the Saved Variables,
	-- or setting up slash commands.
	self.db = LibStub("AceDB-3.0"):New("MetaGuildDB", defaults, true);
	if(self.db.realm.main == nil) then
		self.db.realm.main = UnitName("player");
	end
	if(self.db.realm.rank == nil) then
		self.db.realm.rank = 0
	end
	
	if(self.db.realm.timeOffset == nil or self.db.realm.timeOffset == "" or self.db.realm.rank >= 3) then
		self.db.realm.timeOffset = 0
	end
	
	self.db.RegisterCallback(self, "OnNewProfile", "LoadedNewProfile");
	self.db.RegisterCallback(self, "OnProfileChanged", "LoadedNewProfile");
	self.db.RegisterCallback(self, "OnProfileCopied", "LoadedNewProfile");
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileReset");
	self.db.RegisterCallback(self, "OnProfileShutdown", "LeaveProfile");
	
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	--Prune out the Dev options for non Developers.
	
	if self:GetMainFromName(UnitName("player")) ~= "Visceris" and self:GetMainFromName(UnitName("player")) ~= "Handevon" and UnitName("player") ~= "Senjinn" then
		--self:Print("Not the Dev")
		options.args.dev = nil
	end
	
	icon:Register("MetaGuild", dataobj, MetaGuildDB)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("MetaGuild", options, {"MetaGuild", "MG", "Meta"});
	self.optionsFrame = AceConfigDialog:AddToBlizOptions("MetaGuild", "MetaGuild");
	self:RegisterChatCommand("metaguild", "SlashCommand")
	self:RegisterChatCommand("mg", "SlashCommand")
	self:RegisterChatCommand("meta", "SlashCommand")
	
	--tinsert(UISpecialFrames,"MetaGuild_OnlineFrame");
	
	self:InitRightClickMenu()
	
	self:InitPopupDialogs()
	
	--Set the Version
	self.db.realm.version = version
end


function MG:OnEnable()
	-- Do more initialization here, that really enables the use of your addon.
	-- Register Events, Hook functions, Create Frames, Get information from 
	-- the game that wasn't available in OnInitialize
	if (self.db.profile.enabled == false) then
		return;
	end
	
	-- Wipe the Online Database
	self.db.realm.onlinePlayers = {}
	
	-- Events
	self:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE")
	self:RegisterEvent("CHAT_MSG_CHANNEL_JOIN")
	self:RegisterEvent("CHAT_MSG_CHANNEL_LEAVE")
	self:RegisterEvent("CHAT_MSG_CHANNEL")
	self:RegisterEvent("CHAT_MSG_SYSTEM")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("ACHIEVEMENT_EARNED")
	self:RegisterEvent("PLAYER_LEVEL_UP")
	self:RegisterEvent("TRADE_SKILL_SHOW")
	self:RegisterEvent("CHAT_MSG_CHANNEL_LIST")
	self:RegisterEvent("PVP_RATED_STATS_UPDATE")
	
	self:JoinMetaGuildChannels()

	self:RegisterComm(string.upper(self.db.realm.baseChannel))
	self:Print("Just setup the comm for "..self.db.realm.baseChannel)
--	if self.db.realm.guildChannel then 
		--self:RegisterComm(string.upper(self.db.realm.guildChannel))
--	end
	-- Hooks
	if not self:IsHooked("SendChatMessage") then
		self:RawHook("SendChatMessage", true);
	end
	
	if not self:IsHooked("ChatEdit_UpdateHeader") then
		self:RawHook("ChatEdit_UpdateHeader", true);
	end
	
	if not self:IsHooked("UnitPopup_HideButtons") then
		self:Hook("UnitPopup_HideButtons", true)
	end
	
	if not self:IsHooked("UnitPopup_OnClick") then
		self:SecureHook("UnitPopup_OnClick")
	end	
	
	
	
	--[[if not self:IsHooked("PlayerNameAutocomplete") then
		MG:RawHook("PlayerNameAutocomplete", true);
	end]]--
	
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", MetaGuildChatFilter);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL_NOTICE_USER", MetaGuildChatRemover);
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL_LIST", MetaGuildChatRemover);	


	-->	Set all the GUI Names from the Localization array	<--
	MetaGuildFrameGuildNameLabel:SetText(self.db.realm.guildName)
	MetaGuildFrameMembersCountLabel:SetText(L["Main/MembersOnlineText"])
	
	MetaGuildFrameCreateFrameGuildCreateLabel:SetText(L["Create/Label"])
	MetaGuildFrameCreateFrameGuildCreateText:SetText(L["Create/Text"])
	MetaGuildFrameCreateFrameGuildCreateNameLabel:SetText(L["Create/NameLabel"])
	MetaGuildFrameCreateFrameGuildCreateChannelLabel:SetText(L["Create/ChannelLabel"])
	MetaGuildFrameCreateFrameGuildCreateButton:SetText(L["Create/Button"])
	
	--> GUI Tabs  <--
	MetaGuildFrameTab1:SetText(L["Main/Tab_Guild"])
	MetaGuildFrameTab2:SetText(L["Main/Tab_Roster"])
	MetaGuildFrameTab3:SetText(L["Main/Tab_News"])
	MetaGuildFrameTab4:SetText(L["Main/Tab_Info"])

	
	if(GetChannelName(self.db.realm.guildChannel) > 0) then
		self:OnChannelJoined();
	else
		-- Don't join channel yet, because it will discard messages. It will be done after another channel is joined.
		-- self.joinTimer = self:ScheduleRepeatingTimer("JoinMetaGuildChannels", 3);
	end
	
	_G["MetaGuildFrame"]:SetScale(self.db.profile.scale)
	

	-->  Setup the Tabs  <--
	PanelTemplates_SetNumTabs(MetaGuildFrame, 4)
 	PanelTemplates_SetTab(MetaGuildFrame, 2)      -- We want tab 2 selected.
 	MetaGuildFrameGuildTabFrame:Hide()
 	MetaGuildFrameRosterTabFrame:Show()
 	MetaGuildFrameNewsTabFrame:Hide()
	MetaGuildFrameInfoTabFrame:Hide()
	
	
	self:UpdateOnlineCount()
	

	
	
	--UnitPopupMenus["SELF"] = { "SET_FOCUS", "PVP_FLAG", "LOOT_METHOD", "LOOT_THRESHOLD", "OPT_OUT_LOOT_TITLE", "LOOT_PROMOTE", "CONVERT_TO_RAID", "CONVERT_TO_PARTY", "DUNGEON_DIFFICULTY", "RAID_DIFFICULTY", "RESET_INSTANCES", "RAID_TARGET_ICON", "SELECT_ROLE", "LEAVE", "CANCEL", "METAGUILD_INVITE" };

	--Need to hook UnitPopup_OnClick to add a function check after the main one has been done.
	
	if version ~= self.db.realm.version then
		self:Print("Upgraded to Version "..version)
		displayNewVersion = true
	end

	--self:SendVersionRequest()
	self:Print("v" .. self.db.realm.version .. " loaded")
end

function MG:UnitPopup_HideButtons()
	local k
	--self:Print("Hide buttons has been run")	
	local dropdownMenu = UIDROPDOWNMENU_INIT_MENU
	--self:Print("Type: "..dropdownMenu.which)
	--self:Print("Level: "..UIDROPDOWNMENU_MENU_LEVEL)
	for index, value in ipairs(UnitPopupMenus[UIDROPDOWNMENU_MENU_VALUE] or UnitPopupMenus[dropdownMenu.which]) do
	--for i,v in ipairs(UnitPopupMenus[dropdownMenu.which]) do
		if value == "METAGUILD_INVITE" then 
			k = (not self:CanInvite(dropdownMenu.name) or dropdownMenu.name == UnitName("player")) and 0 or 1
			--self:Print("invite: "..k)
			UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = k
		elseif value == "METAGUILD_KICK" then
			k = (not self:CanKick(dropdownMenu.name) or dropdownMenu.name == UnitName("player")) and 0 or 1
			--self:Print("kick: "..k)
			UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = k
		--elseif rightClickFunctions[value] then 
		--	UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = (dropdownMenu.name == UnitName("player") and 0) or 1
		end
	end	
end

function MG:UnitPopup_OnClick(widget)
	local button = widget.value
	local dropdownFrame = UIDROPDOWNMENU_INIT_MENU
	local name = dropdownFrame.name

	--self:Print("Clicking Menu Item: "..button)
	if button == "METAGUILD_INVITE" then
		self:Print("Trying to Invite: "..name)
		self:SendInvite(name)
	elseif button == "METAGUILD_KICK" then
		StaticPopup_Show("REMOVE_METAGUILDMEMBER")
	elseif button == "METAGUILD_SYNC" then
		self:SyncDatabaseWithPlayer(name)
	end
	
	--
	PlaySound("UChatScrollButton")
end

function MetaGuildInvite(name)
	self:SendInvite(name)
end

function MetaGuildKick(name)
	self:SendKick(name)
end


function MG:InitRightClickMenu()
	local function insertbefore(t, before, val)
		for k,v in ipairs(t) do
			if v == before then 
				return table.insert(t, k, val)
			end 
		end
		table.insert(t, val)
	end

	UnitPopupButtons["METAGUILD_INVITE"] = { text = "MetaGuild Invite", dist = 0 }
	UnitPopupButtons["METAGUILD_KICK"] = { text = "MetaGuild Kick", dist = 0 }
	UnitPopupButtons["METAGUILD_SYNC"] = { text = "MetaGuild Sync", dist = 0 }

	rightClickFunctions = {["METAGUILD_INVITE"] = MetaGuildInvite, ["METAGUILD_KICK"] = MetaGuildKick}

	insertbefore(UnitPopupMenus["PLAYER"], "CANCEL", "METAGUILD_INVITE")
	insertbefore(UnitPopupMenus["RAID_PLAYER"], "CANCEL", "METAGUILD_INVITE")	
	insertbefore(UnitPopupMenus["PARTY"], "CANCEL", "METAGUILD_INVITE")
	insertbefore(UnitPopupMenus["FRIEND"], "CANCEL", "METAGUILD_INVITE")
	
	insertbefore(UnitPopupMenus["PLAYER"], "CANCEL", "METAGUILD_SYNC")
	insertbefore(UnitPopupMenus["RAID_PLAYER"], "CANCEL", "METAGUILD_SYNC")	
	insertbefore(UnitPopupMenus["FRIEND"], "CANCEL", "METAGUILD_SYNC")
	insertbefore(UnitPopupMenus["PARTY"], "CANCEL", "METAGUILD_SYNC")
	insertbefore(UnitPopupMenus["SELF"], "CANCEL", "METAGUILD_SYNC")

end


function MG:JoinMetaGuildChannels()
	--This will allow us to join the private guild channel.
	--It will use the password that came with the 
	valid = true
	
	--self:Print("Guild Channel Value: "..GetChannelName(self.db.realm.guildChannel))
	
	if self.db.realm.guildChannel ~= nil then
		if(GetChannelName(self.db.realm.guildChannel) <= 0) then
			JoinChannelByName(self.db.realm.guildChannel, self.db.realm.guildPassword);
			if(GetChannelName(self.db.realm.guildChannel) <= 0) then
				valid = false
			else
				--self:OnChannelJoined()
			end
		end
	end

	if valid then
		self:CancelAllTimers()
		
	end
end

function MG:GetZone()
	--self:Print(GetZoneText())
	zone = GetRealZoneText()
	if (zone == nil) then
		zone = GetZoneText()
	end
	if zone == nil then
		zone = "Unknown"
	end
	return zone
end

function MG:OnChannelJoined()
	--self:CancelTimer(self.joinTimer, true);
	self:CancelAllTimers();
	--self:SetSlashCmd();
	
	-- Chat magic
	addonNameUpper = self.db.realm.baseChannel;
	ChatTypeInfo[addonNameUpper] = ChatTypeInfo["CHANNEL"..GetChannelName(self.db.realm.guildChannel)];
    	ChatTypeInfo[addonNameUpper].sticky = 1;
	_G["CHAT_MSG_" .. addonNameUpper] = "metaguild";
	_G["CHAT_" .. addonNameUpper .. "_GET"] = "[MetaGuild] %s:\32";
	_G["CHAT_" .. addonNameUpper .. "_SEND"] = "MetaGuild:\32";
	
	ListChannelByName(self.db.realm.guildChannel);
	
	local frames = self:FindChatFrames()
	if not next(frames) then
		-- The channel should be assigned to at last one frame to make sense.
		MG:Print("MetaGuild is not assigned to a chat frame. Assigning to default chat frame.");

		if self.db.realm.guildChannel then
			ChatFrame_AddChannel(DEFAULT_CHAT_FRAME, self.db.realm.guildChannel);
		end
	end
	
	ChangeChatColor("CHANNEL" .. GetChannelName(self.db.realm.guildChannel), self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b); 

	-- TODO: Sometimes these messages aren't sent.
	
	--zone = MG:GetZone()
	--self:Print("Player Level: ".. UnitLevel("player"))
	--self:Print("Zone: ".. self:GetZone())
	--self:Print("Rank: ".. self.db.realm.rank)
	
	self:SendJoin()
	self:SendPvpUpdate(1)
	
	if self.db.profile.autoSync then
		self:StartSyncDatabase()
	end
	
end

function MG:OnChannelLeft()
	--self:RemoveSlashCmd();
	
	-- Chat magic
	addonNameUpper = self.db.realm.baseChannel;	
	ChatTypeInfo[addonNameUpper] = nil
	_G["CHAT_MSG_" .. addonNameUpper] = nil;
	_G["CHAT_" .. addonNameUpper .. "_GET"] = nil;
	_G["CHAT_" .. addonNameUpper .. "_SEND"] = nil;
	
	DEFAULT_CHAT_FRAME.editBox:SetAttribute("stickyType", "SAY");
	DEFAULT_CHAT_FRAME.editBox:SetAttribute("chatType", "SAY");
	--_G["ChatFrame" .. self.db.profile.chatFrame].editBox:SetAttribute("stickyType", "SAY");
	--_G["ChatFrame" .. self.db.profile.chatFrame].editBox:SetAttribute("chatType", "SAY");
	
	hash_SlashCmdList = {}
	hash_ChatTypeInfoList = {}
end


function MG:OnDisable()
	-- Unhook, Unregister Events, Hide frames that you created.
	-- You would probably only use an OnDisable if you want to 
	-- build a "standby" mode, or be able to toggle modules on/off.
	
	if(self.db.profile.enabled == true) then
		return
	end
	
	self:UnregisterComm(string.upper(self.db.realm.baseChannel))
	
	if self.db.realm.GuildChannel then 
		LeaveChannelByName(self.db.realm.guildChannel);
	end
	
	self:Unhook("SendChatMessage", true)
	self:Unhook("ChatEdit_UpdateHeader", true)
	self:Unhook("UnitPopup_ShowMenu", true)
	self:Unhook("UnitPopup_OnClick", true)
	self:Unhook("UnitPopup_HideButtons", true)
	
	
	
	self.db.realm.onlinePlayers = {}
	numOnline = 0
	self:UpdateOnlineCount()
end

function MG:CanSend()
	if (self.db.profile.enabled) then
		if(self.db.realm.guildChannel ~= nil and self.db.realm.guildPassword ~= nil) then
			if (GetChannelName(self.db.realm.guildChannel) > 0) then
				return true;
			end
		else
			self:Print("Could not send. Guild Channel & Password are not set correctly")
		end
	else
		self:Print("Could not send. Addon Disabled")
	end
	return false;
end

-- EVENTS --

function MG:PVP_RATED_STATS_UPDATE(eventName, ...)
	--self:Print("Updated PVP Stats")
	self:SendPvpUpdate(0)
end

function MG:CHAT_MSG_CHANNEL_NOTICE(eventName, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	-- Join/Leave channel
	if(arg1 == "YOU_JOINED" and arg9 == self.db.realm.guildChannel) then
		if(self.db.profile.enabled) then
			self:OnChannelJoined();
		end
	elseif(arg1 == "YOU_LEFT" and arg9 == self.db.realm.guildChannel) then
		--if(self.db.profile.enabled) then
			self:OnChannelLeft();
			self.db.profile.enabled = false;
			self:OnDisable();
		--end
	elseif(self.db.profile.enabled) then
		self.joinTimer = self:ScheduleRepeatingTimer("JoinMetaGuildChannels", 2);
	end
end

function MG:CHAT_MSG_CHANNEL_JOIN(eventName, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	if(arg9 == self.db.realm.guildChannel) then
		local name = arg2;
		--ListChannelByName(self.db.realm.guildChannel);
		if(self.db.realm.onlinePlayers[name] == nil) then
			--self.db.realm.onlinePlayers[name] = { ["main"] = "", ["status"] = 4}
		end
		--self:Print("Updating MetaGuild from Channel Join")
		self:SendUpdate();
		self:UpdateOnlineCount();
	end
end

function MG:CHAT_MSG_CHANNEL_LEAVE(eventName, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	if(arg9 == self.db.realm.guildChannel and arg2 ~= UnitName("player")) then
		local name = arg2;
		--self:SystemMessage(player .. " left");
		self.db.realm.onlinePlayers[name] = nil;
		self:UpdateOnlineCount()
		
		local infoFrame = getglobal("MetaGuild_PlayerInfoFrame")
		local RosterFrame = getglobal("MetaGuildRosterFrame")
		if infoFrame:IsShown() and name == infoFrame.name then
			infoFrame:Hide();
			RosterFrame.selectedGuildMember = nil
		end
		
	end
end

function MG:CHAT_MSG_CHANNEL(eventName, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	if(arg9 == MG.db.realm.guildChannel) then
		if(string.sub(arg1, 0, 1) == specialChar) then
			text = self:ParseMessage(arg2, string.sub(arg1, 1))
			
			if(text) then
				self:SystemMessage(text)
			end
		end
	end
end

function MG:CHAT_MSG_SYSTEM(eventName, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	-- AFK/DND status changed
	msg = arg1;
	local initStatus = status
	if string.find(msg, L["AfkMessage"]) then
		status = 2;
	elseif string.find(msg, L["RemoveAfkMessage"]) then
		status = 1;
	elseif string.find(msg, L["DndMessage"]) then
		status = 3;
	elseif string.find(msg, L["RemoveDndMessage"]) then
		status = 1;
	end
	--self:Print("Printing MetaGuild from Chat Message: "..msg)
	if status ~= initStatus then
		self:SendUpdate();
	end
end

function MG:ZONE_CHANGED_NEW_AREA()
	-- Zone changed
	self:SendUpdate()
end


function MG:ACHIEVEMENT_EARNED(eventName, id)
	--self:Print("Achiev Id: "..id)
	if id ~= nil and not select(12,GetAchievementInfo(id)) then
		
	--local link = GetAchievementLink(id)
	--link = self:removeLinks(link)
	--self:Print("About to send Link: "..link)
	--SendChatMessage(specialChar .. "ACHIEVEMENT" .. specialChar .. link .. specialChar, self.db.realm.guildChannel)
		self:SendAchievement(id)
	end
	

end

function MG:TRADE_SKILL_SHOW(eventName, ...)
	if not TradeSkillFrameTitleText then 
			return 
	end
	
	if (IsTradeSkillLinked()) then
		return
	end
	
	if (GetTradeSkillListLink() == nil) then
		return
	end
	
	local tradeskillName, currentLevel, maxLevel = GetTradeSkillLine();
	if (self.db.realm.professions[tradeskillName] == nil or self.db.realm.professions[tradeskillName].skill <= currentLevel) then
		self.db.realm.professions[tradeskillName] = {};
		self.db.realm.professions[tradeskillName].link = GetTradeSkillListLink();
		self.db.realm.professions[tradeskillName].skill = currentLevel;
	end
	if (self.db.char.professions[tradeskillName] == nil) or (self.db.char.professions[tradeskillName].skill <= currentLevel) then
		self.db.char.professions[tradeskillName] = {};
		self.db.char.professions[tradeskillName].link = GetTradeSkillListLink();
		self.db.char.professions[tradeskillName].skill = currentLevel;
	end
end

function MG:PLAYER_LEVEL_UP(eventName, ...)
	local level = select(1,...)
	self:SendLevelUp(level)
	--Send a level up msg.
end

function MG:CHAT_MSG_CHANNEL_LIST(eventName, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	if(arg9 == self.db.realm.guildChannel) then
		for k,v in pairs(self:explode(", ", arg1)) do
			if(string.sub(v, 0, 1) == "*") then
				v = string.sub(v, 2);
			end
			if(self.db.realm.onlinePlayers[v] == nil) then
				--self:Print("Just got a msg from a member that is not on the Online list ("..v..")")
				--self.db.realm.onlinePlayers[v] = { ["main"] = "", ["status"] = 4}
			end
		end
		self:UpdateOnlineCount()
	end
end

-- HOOKS --
function MG:SendChatMessage(msg, chan, language, target)
	--self:Print("Chan: "..chan)
	local channelId = GetChannelName(self.db.realm.guildChannel)
	if((channelId ~= 0) and (chan == self.db.realm.guildChannel)) then
		chan = "CHANNEL";
		target = channelId;
		--if(string.sub(msg, 0, 1) ~= specialChar) then
			msg = self:removeLinks(msg);
		--end
	end
	--print("Sending Chat Message: "..msg)
	self.hooks["SendChatMessage"](msg, chan, language, target)
end

function MG:ChatEdit_UpdateHeader(editBox)
	self.hooks["ChatEdit_UpdateHeader"](editBox);
	
	addonNameUpper = self.db.realm.baseChannel;
	if (editBox:GetAttribute("chatType") == addonNameUpper) then
		editBox:SetTextColor(self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b);
		_G[editBox:GetName().."Header"]:SetTextColor(self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b);
    end
end

function MetaGuildChatFilter(ChatFrameSelf, event, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12 = ...;
	if(arg9 == MG.db.realm.guildChannel and MG.db.profile.filterDisabled == false) then
		if(string.sub(arg1, 0, 1) == specialChar) then
			return true;
		else
			arg1 = self:addLinks(arg1);
			arg1 = self:resolveSpecial(arg1);
			return false, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12;
		end
	end
	
	return false, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12;
end

function MetaGuildChatRemover(ChatFrameSelf, event, ...)
	arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9 = ...;
	--self:Print("Arg: "..arg9)
	if MG.db.profile.filterDisabled then
		--Disabled the chat filter. Addon messages should now be shown.
		return false
	else
		if(arg9 == MG.db.realm.guildChannel) then
			return true
		end
	end
end

function MG:FindChatFrames()
	local frames = {}
	for i = 1,10 do
		for index,value in pairs({GetChatWindowChannels(i)}) do
			if (index%2==1) and (value == self.db.realm.guildChannel) then
				tinsert(frames, i);
			end
		end
	end
	return frames
end

function MG:SystemMessage(msg)
	--self:Print("Sending Message: "..msg)
	
	local tagName = self.db.profile.chatTag
	if self.db.realm.guildName ~= nil and self.db.realm.guildName ~= "" then
		tagName = self.db.realm.guildName
	end
	for i,v in pairs(self:FindChatFrames()) do
		_G["ChatFrame" .. v]:AddMessage("[" .. tagName .. "] " .. msg, self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b);
	end
end

function MG:IsValidDataType(dataType, targetTypes)
	
	if dataType == "ALL" then
		return true
	else
		for k,v in ipairs(targetTypes) do
			if dataType == v then
				return true
			end
		end
	end
	return false
end

function MG:DataSyncRequest(sender, data, dataType)
	--This function will check our data against the packed data.
	--If there is no packed data, then we will just assume the data is all newer than ours and include our entire database dates, or just the data type if its supplied.
	--This function will spawn "Request" messages if the data is newer than ours or "Send"

	--Request data will always be made up of just time codes. No actual data will be sent in these messages.

	--It is important to keep the data structure the same as the database.
	
	--DataType allows you to request only one type of Data.	
	
	if (self:CanSend()) then
	
		local sendData = {}
		local reqData = {}
		local forceReq

		if data ~= nil then
			forceReq = false
		else
			data = {}
			forceReq = true
			self:Print("Including all data")
		end

		if dataType == nil then
			dataType = "ALL"
		end


		--Check the MOTD
		if self:IsValidDataType(dataType, {"MOTD"}) then
			--self:Print("Got into MotD")
			if self.db.realm.motd == nil or self.db.realm.motd.time == nil then
				--Setup the motd if it doesnt yet exist.
				self.db.realm.motd = {["time"] = 0}
			end
			if forceReq or (data.motd ~= nil and data.motd.time > self.db.realm.motd.time) then
				--self:Print("Adding MotD to Request")
				reqData.motd = {["time"] = self.db.realm.motd.time}
			else	
				if data.motd ~= nil and data.motd.time ~= self.db.realm.motd.time then
					--self:Print("Adding MotD to Send")
					sendData.motd = self.db.realm.motd
				end
			end
		end

		--Check the Mission Statement
		if self:IsValidDataType(dataType, {"MISSION"}) then
			--self:Print("Got into Mission")
			if self.db.realm.mission == nil or self.db.realm.mission.time == nil then
				--Setup the mission if it doesnt yet exist.
				self.db.realm.mission = {["time"] = 0}
			end
			if forceReq or (data.mission ~= nil and data.mission.time > self.db.realm.mission.time) then
				reqData.mission = {["time"] = self.db.realm.mission.time}
				--self:Print("Adding Mission Statement to Request")
			else
				if data.mission ~= nil and data.mission.time ~= self.db.realm.mission.time then
					sendData.mission = self.db.realm.mission
					--self:Print("Adding Mission Statement to Send")
				end
			end
		end

		--Check the Guild Info
		if self:IsValidDataType(dataType, {"INFO"}) then
			if self.db.realm.info == nil or self.db.realm.info.time == nil then
				--Setup the info if it doesnt yet exist.
				self.db.realm.info = {["time"] = 0}
			end
			if forceReq or (data.info ~= nil and data.info.time > self.db.realm.info.time) then
				reqData.info = {["time"] = self.db.realm.info.time}
				--self:Print("Adding Guild Info to Request")
			else
				if data.info ~= nil and data.info.time ~= self.db.realm.info.time then
					sendData.info = self.db.realm.info
					--self:Print("Adding Guild Info to Send")
				end
			end
		end


		--Characters will be more difficult due to their many nested levels.
		--Even though it might be a bit less optimal, we will add all the mains first
		--Then after they are all added, add the characters to each main.
		--This will also ensure that the public/officer/pvpdata componants to each character will have the needed hierchy setup before adding their data.
		if self:IsValidDataType(dataType, {"CHARACTER", "PUBLIC", "OFFICER", "PVP"}) then
			--We will loop through our mains first. If any of them are different we will add them to the correct array.
			--Afterwards we will add any of the "New" mains from the data unpacked data.
			local mains = {}
			for main,b in pairs(self.db.realm.offlinePlayers) do
				if not tContains(mains, main) then
					tinsert(mains, main)
				end
			end

			if not forceReq and data.offlinePlayers ~= nil then
				for main,b in pairs(data.offlinePlayers) do
					if not tContains(mains, main) then
						tinsert(mains, main)
					end
				end
			end

			for _,main in pairs(mains) do
				-- "b" is the name of the main character we are looking at.

				--self:Print("Current Main: "..main)

				--We need to put the requested characters and the offline characters into the same list
				--That will represent all the characters that we know of and we will be able to test the existance of attributes as we proccess the list.
				--If you find something that we do not have in our offline database, then add it to the request. 
				--If we have something that isnt in the datarequest, then add it to the send msg.

				--Make sure we have this main in our database. If not, put in a place holder which will be updated shortly.
				if self.db.realm.offlinePlayers[main] == nil then
					self.db.realm.offlinePlayers[main] = {["characters"] = {}, ["time"]=0}
				end

				if forceReq or data.offlinePlayers ~= nil and data.offlinePlayers[main] ~= nil and data.offlinePlayers[main].time > self.db.realm.offlinePlayers[main].time then
					if reqData.offlinePlayers == nil then
						reqData.offlinePlayers = {}
					end
					reqData.offlinePlayers[main] = {["characters"] = {}, ["time"]=self.db.realm.offlinePlayers[main].time}
				else
					if data.offlinePlayers ~= nil and data.offlinePlayers[main] ~= nil and data.offlinePlayers[main].time ~= self.db.realm.offlinePlayers[main].time then
						--send them the new data
						if sendData.offlinePlayers == nil then
							sendData.offlinePlayers = {}
						end				
						sendData.offlinePlayers[main] = {["characters"] = {}, ["time"]=self.db.realm.offlinePlayers[main].time, ["rank"]=self.db.realm.offlinePlayers[main].rank}	
					else
						--Just skip it since the records would equal eachother.
					end
				end

				--At this point, the main will exist in our offline database, but might not have any character data.
				--We will go through all the charactes for this main.
				--Again, we will need to create a character table that contains all the characters for this main in both the offline database and the request data.
				local characters = {}
				for name,b in pairs(self.db.realm.offlinePlayers[main].characters) do
					if not tContains(characters, name) then
						tinsert(characters, name)
					end
				end

				if not forceReq and data.offlinePlayers ~= nil and data.offlinePlayers[main] ~= nil and data.offlinePlayers[main].characters ~= nil then
					for name,b in pairs(data.offlinePlayers[main].characters) do
						if not tContains(characters, name) then
							--self:Print("Adding "..name.." to the character table")
							tinsert(characters, name)
						end
					end
				end

				--Ok, we have a list of all the known characters that exist for this main.

				--self:Print("Character Count: "..#characters)
				for _,name in ipairs(characters) do

					--self:Print("Current Character: "..name)
					--Make sure we have this character in our database. If not, put in a place holder which will be updated shortly.
					if self:IsValidDataType(dataType, {"CHARACTER"}) then
						if self.db.realm.offlinePlayers[main].characters[name] == nil then
							self.db.realm.offlinePlayers[main].characters[name] = {["time"]=0}
						end
						if forceReq or data.offlinePlayers ~= nil and data.offlinePlayers[main] ~= nil and data.offlinePlayers[main].characters[name] ~= nil and data.offlinePlayers[main].characters[name].time > self.db.realm.offlinePlayers[main].characters[name].time then
							--It is possible at this point that the main info has not changes, but the character info needs to be updated.
							--If this is the case the offlinePlayers table would not have been created yet. Make it here.

							if reqData.offlinePlayers == nil then
								reqData.offlinePlayers = {}
							end

							if reqData.offlinePlayers[main] == nil then
								--self:Print("Adding "..main.." to request")
								reqData.offlinePlayers[main] = {["characters"] = {}, ["time"]=self.db.realm.offlinePlayers[main].time}
							end
							--self:Print("Adding Character: "..name.." to request")
							reqData.offlinePlayers[main].characters[name] = {["time"]=self.db.realm.offlinePlayers[main].characters[name].time}
						else
							if data.offlinePlayers ~= nil and data.offlinePlayers[main] ~= nil and data.offlinePlayers[main].characters[name] ~= nil and data.offlinePlayers[main].characters[name].time ~= self.db.realm.offlinePlayers[main].characters[name].time then
								--send them the new character data
								--Again, if the character data is different, but the main data wasnt, we would need to add the main data to the sendData table.
								if sendData.offlinePlayers == nil then
									sendData.offlinePlayers = {}
								end

								if sendData.offlinePlayers[main] == nil then
									sendData.offlinePlayers[main] = {["characters"] = {}, ["time"]=self.db.realm.offlinePlayers[main].time, ["rank"]=self.db.realm.offlinePlayers[main].rank, ["joined"]=self.db.realm.offlinePlayers[main].joined}
								end

								sendData.offlinePlayers[main].characters[name] = {["time"]=self.db.realm.offlinePlayers[main].characters[name].time, ["zone"]=self.db.realm.offlinePlayers[main].characters[name].zone, ["level"]=self.db.realm.offlinePlayers[main].characters[name].level, ["class"]=self.db.realm.offlinePlayers[main].characters[name].class}

							else
								--Just skip it since the records would equal eachother.
							end
						end
					end

					--Now for the public notes
					if self:IsValidDataType(dataType, {"PUBLIC"}) then
						if self.db.realm.offlinePlayers[main].characters[name].public == nil then
							self.db.realm.offlinePlayers[main].characters[name].public = {["time"]=0}
						end

						if forceReq or data.offlinePlayers ~= nil and data.offlinePlayers[main] ~= nil and data.offlinePlayers[main].characters[name] ~= nil and data.offlinePlayers[main].characters[name].public ~= nil and data.offlinePlayers[main].characters[name].public.time > self.db.realm.offlinePlayers[main].characters[name].public.time then

							if reqData.offlinePlayers == nil then
								reqData.offlinePlayers = {}
							end

							if reqData.offlinePlayers[main] == nil then
								reqData.offlinePlayers[main] = {["characters"] = {}, ["time"]=self.db.realm.offlinePlayers[main].time}
							end

							if reqData.offlinePlayers[main].characters[name] == nil then
								reqData.offlinePlayers[main].characters[name] = {["time"]=self.db.realm.offlinePlayers[main].characters[name].time}
							end

							reqData.offlinePlayers[main].characters[name].public = {["time"]=self.db.realm.offlinePlayers[main].characters[name].public.time}
						else
							if data.offlinePlayers ~= nil and data.offlinePlayers[main] ~= nil and data.offlinePlayers[main].characters[name] ~= nil and data.offlinePlayers[main].characters[name].public ~= nil and data.offlinePlayers[main].characters[name].public.time ~= self.db.realm.offlinePlayers[main].characters[name].public.time then

								if sendData.offlinePlayers == nil then
									sendData.offlinePlayers = {}
								end

								if sendData.offlinePlayers[main] == nil then
									sendData.offlinePlayers[main] = {["characters"] = {}, ["time"]=self.db.realm.offlinePlayers[main].time, ["rank"]=self.db.realm.offlinePlayers[main].rank}
								end

								if sendData.offlinePlayers[main].characters[name] == nil then
									sendData.offlinePlayers[main].characters[name] = {["time"]=self.db.realm.offlinePlayers[main].characters[name].time}
								end

								sendData.offlinePlayers[main].characters[name].public = self.db.realm.offlinePlayers[main].characters[name].public
							else
								--Just skip it since the records would equal eachother.
							end
						end
					end


					--Now for the officer notes
					if self:IsValidDataType(dataType, {"OFFICER"}) then
						if self.db.realm.offlinePlayers[main].characters[name].officer == nil then
							self.db.realm.offlinePlayers[main].characters[name].officer = {["time"]=0}
						end

						if forceReq or data.offlinePlayers ~= nil and data.offlinePlayers[main] ~= nil and data.offlinePlayers[main].characters[name] ~= nil and data.offlinePlayers[main].characters[name].officer ~= nil and data.offlinePlayers[main].characters[name].officer.time > self.db.realm.offlinePlayers[main].characters[name].officer.time then

							if reqData.offlinePlayers == nil then
								reqData.offlinePlayers = {}
							end

							if reqData.offlinePlayers[main] == nil then
								reqData.offlinePlayers[main] = {["characters"] = {}, ["time"]=self.db.realm.offlinePlayers[main].time}
							end

							if reqData.offlinePlayers[main].characters[name] == nil then
								reqData.offlinePlayers[main].characters[name] = {["time"]=self.db.realm.offlinePlayers[main].characters[name].time}
							end

							reqData.offlinePlayers[main].characters[name].officer = {["time"]=self.db.realm.offlinePlayers[main].characters[name].officer.time}
						else
							if data.offlinePlayers ~= nil and data.offlinePlayers[main] ~= nil and data.offlinePlayers[main].characters[name] ~= nil and data.offlinePlayers[main].characters[name].officer ~= nil and data.offlinePlayers[main].characters[name].officer.time ~= self.db.realm.offlinePlayers[main].characters[name].officer.time then

								if sendData.offlinePlayers == nil then
									sendData.offlinePlayers = {}
								end

								if sendData.offlinePlayers[main] == nil then
									sendData.offlinePlayers[main] = {["characters"] = {}, ["time"]=self.db.realm.offlinePlayers[main].time, ["rank"]=self.db.realm.offlinePlayers[main].rank}
								end

								if sendData.offlinePlayers[main].characters[name] == nil then
									sendData.offlinePlayers[main].characters[name] = {["time"]=self.db.realm.offlinePlayers[main].characters[name].time}
								end

								sendData.offlinePlayers[main].characters[name].officer = self.db.realm.offlinePlayers[main].characters[name].officer
							else
								--Just skip it since the records would equal eachother.
							end
						end
					end

					--Now for the pvp stats
					if self:IsValidDataType(dataType, {"PVP"}) then
						if self.db.realm.offlinePlayers[main].characters[name].pvp == nil then
							self.db.realm.offlinePlayers[main].characters[name].pvp = {["time"]=0}
						end
						
						if self.db.realm.offlinePlayers[main].characters[name].pvp.time == nil then
							self.db.realm.offlinePlayers[main].characters[name].pvp.time =0
						end
						--self:Print("Main: "..main..", Character: "..name)
						
						--self:Print(self.db.realm.offlinePlayers[main].characters[name].pvp.time)
						
						
						if forceReq or (data.offlinePlayers ~= nil and data.offlinePlayers[main] ~= nil and data.offlinePlayers[main].characters[name] ~= nil and data.offlinePlayers[main].characters[name].pvp ~= nil and tonumber(data.offlinePlayers[main].characters[name].pvp.time) > tonumber(self.db.realm.offlinePlayers[main].characters[name].pvp.time)) then
							if reqData.offlinePlayers == nil then
								reqData.offlinePlayers = {}
							end

							if reqData.offlinePlayers[main] == nil then
								reqData.offlinePlayers[main] = {["characters"] = {}, ["time"]=self.db.realm.offlinePlayers[main].time}
							end

							if reqData.offlinePlayers[main].characters[name] == nil then
								reqData.offlinePlayers[main].characters[name] = {["time"]=self.db.realm.offlinePlayers[main].characters[name].time}
							end

							reqData.offlinePlayers[main].characters[name].pvp = {["time"]=self.db.realm.offlinePlayers[main].characters[name].pvp.time}
						else
							if (data.offlinePlayers ~= nil and data.offlinePlayers[main] ~= nil and data.offlinePlayers[main].characters[name] ~= nil and data.offlinePlayers[main].characters[name].pvp ~= nil and data.offlinePlayers[main].characters[name].pvp.time ~= self.db.realm.offlinePlayers[main].characters[name].pvp.time) then
								--self:Print(main..":"..name)
								if sendData.offlinePlayers == nil then
									sendData.offlinePlayers = {}
								end

								if sendData.offlinePlayers[main] == nil then
									sendData.offlinePlayers[main] = {["characters"] = {}, ["time"]=self.db.realm.offlinePlayers[main].time, ["rank"]=self.db.realm.offlinePlayers[main].rank}
								end
								if sendData.offlinePlayers[main].characters[name] == nil then
									sendData.offlinePlayers[main].characters[name] = {["time"]=self.db.realm.offlinePlayers[main].characters[name].time}
								end								

								sendData.offlinePlayers[main].characters[name].pvp = self.db.realm.offlinePlayers[main].characters[name].pvp
							else
								--Just skip it since the records would equal eachother.
							end
						end
					end			
				end			
			end
		end
		
		
		--Check the News
		if self:IsValidDataType(dataType, {"NEWS", "ACHIEVEMENT", "LEVEL", "MEMBERSHIP"}) then
			if self.db.realm.news == nil then
				--Setup the news if it doesnt yet exist.
				self.db.realm.news = {["time"]=0, ["achievements"]={}, ["levels"]={}, ["memberships"]={}}
			end
			
			if self.db.realm.news.time == nil then
				self.db.realm.news.time = 0
			end

			if forceReq or (data.news ~= nil and data.news.time > self.db.realm.news.time) then
				reqData.news = {["time"] = self.db.realm.news.time}
				--self:Print("Adding News Time to Request")
			else
				if data.news ~= nil and data.news.time ~= self.db.realm.news.time then
					if sendData.news == nil then
						sendData.news = {["time"]=0}
					end
					sendData.news.time = self.db.realm.news.time
					--self:Print("Adding News Time to Send")
				end
			end
			
			--Now that we have the base news array setup, check its sub-tables

			--News:Achievements
			if self:IsValidDataType(dataType, {"ACHIEVEMENT"}) then
				if self.db.realm.news.achievements == nil then
					--Setup the news if it doesnt yet exist.
					self.db.realm.news.achievements = {}
				end

				--Loop through the achievements now and check which ones are newer.


				--Get a list of all the acheivments
				local achievements = {}
				for _,achiev in ipairs(self.db.realm.news.achievements) do
					--loop through the achievements
					local isNew = true
					for k,v in pairs(achievements) do
						if v.name == achiev.name and v.data == achiev.data and v.time == achiev.time then
							isNew = false
							break
						end
					end
					if isNew then
						--self:Print("Adding DB "..achiev.name..":"..achiev.data.." to list of achievements")
						tinsert(achievements, achiev)
					end
				end

				if not forceReq and data.news ~= nil and data.news.achievements ~= nil then
					--self:Print("Building list from Data")
					for _,achiev in ipairs(data.news.achievements) do
						--self:Print(achiev.name.." "..achiev.data.." "..achiev.time)
						local isNew = true
						for k,v in pairs(achievements) do
							if v.name == achiev.name and v.data == achiev.data and v.time == achiev.time then
								
								isNew = false
								break
							end
						end
						if isNew then
							--self:Print("Adding Data "..achiev.name..":"..achiev.data.." to list of achievements")
							tinsert(achievements, achiev)
						end
					end
				end
				
				
				--self:Print("Total Achievements: "..#achievements)
				for _,achievement in ipairs(achievements) do
					--If this is being forced, or the achievement time is more recent than our time, or it doesnt exist, then 
					--self:Print("Name: "..achievement.name)	
					local index = nil


					--Check to see if the achievement is in the database yet.
					local newEntry = true
					for i,v in ipairs(self.db.realm.news.achievements) do
						if achievement.name == v.name and achievement.data == v.data then
							newEntry = false
							index = i
							break
						end
					end

					if forceReq or newEntry or index ~= nil and achievement.time > self.db.realm.news.achievements[index].time then
						if reqData.news == nil then
							reqData.news = {["time"] = self.db.realm.news.time}
						end

						if reqData.news.achievements == nil then
							reqData.news.achievements = {}
						end
						--self:Print("Adding Achievement to request")
						--Set the time to 0 so it will force an update to be sent.
						tinsert(reqData.news.achievements, {["time"]=0, ["name"]=achievement.name, ["data"]=achievement.data})
					else
						if index ~= nil and self.db.realm.news.achievements[index].time ~= achievement.time then
							--self:Print("Building SendData")
							--send them the new character data
							--Again, if the character data is different, but the main data wasnt, we would need to add the main data to the sendData table.
							if sendData.news == nil then
								sendData.news = {["time"]=data.news.time}
							end

							if sendData.news.achievements == nil then
								sendData.news.achievements = {}
							end
							--self:Print("Adding Achievement to send")
							tinsert(sendData.news.achievements, achievement)
						else
							--Just skip it since the records would equal eachother.
							--self:Print("Record already matches DB. Skipping ("..index..")")
						end
					end
					--end
				end
			end
			


			--News:Levels
			if self:IsValidDataType(dataType, {"LEVEL"}) then
				if self.db.realm.news.levels == nil then
					--Setup the levels if it doesnt yet exist.
					self.db.realm.news.levels = {}
				end

				--Loop through the levels now and check which ones are newer.


				--Get a list of all the levels
				local levels = {}
				for _,level in ipairs(self.db.realm.news.levels) do
					--loop through the levels
					local isNew = true
					for k,v in pairs(levels) do
						if v.name == level.name and v.data == level.data and v.time == level.time then
							isNew = false
							break
						end
					end
					if isNew then
						--self:Print("Adding DB "..level.name..":"..level.data.." to list of levels")
						tinsert(levels, level)
					end
				end

				if not forceReq and data.news ~= nil and data.news.levels ~= nil then
					--self:Print("Building list from Data")
					for _,level in ipairs(data.news.levels) do
						--self:Print(level.name.." "..level.data.." "..level.time)
						local isNew = true
						for k,v in pairs(levels) do
							if v.name == level.name and v.data == level.data and v.time == level.time then
								
								isNew = false
								break
							end
						end
						if isNew then
							--self:Print("Adding Data "..level.name..":"..level.data.." to list of levels")
							tinsert(levels, level)
						end
					end
				end
				
				
				--self:Print("Total levels: "..#levels)
				for _,level in ipairs(levels) do
					--If this is being forced, or the achievement time is more recent than our time, or it doesnt exist, then 
					--self:Print("Name: "..level.name)	
					local index = nil

					--Check to see if the achievement is in the database yet.
					local newEntry = true
					for i,v in ipairs(self.db.realm.news.levels) do
						if level.name == v.name and level.data == v.data then
							newEntry = false
							index = i
							break
						end
					end

					if forceReq or newEntry or index ~= nil and level.time > self.db.realm.news.levels[index].time then
						if reqData.news == nil then
							reqData.news = {["time"] = self.db.realm.news.time}
						end

						if reqData.news.levels == nil then
							reqData.news.levels = {}
						end
						--self:Print("Adding Levels to request")
						--Set the time to 0 so it will force an update to be sent.
						tinsert(reqData.news.levels, {["time"]=0, ["name"]=level.name, ["data"]=level.data})
					else
						if index ~= nil and self.db.realm.news.levels[index].time ~= level.time then
							--self:Print("Building SendData")
							--send them the new character data
							--Again, if the character data is different, but the main data wasnt, we would need to add the main data to the sendData table.
							if sendData.news == nil then
								sendData.news = {["time"]=data.news.time}
							end

							if sendData.news.levels == nil then
								sendData.news.levels = {}
							end
							
							--self:Print("Adding Level to send")
							tinsert(sendData.news.levels, level)
						else
							--Just skip it since the records would equal eachother.
							--self:Print("Record already matches DB. Skipping ("..index..")")
						end
					end
				end
			end
			
			
--News:Levels
			if self:IsValidDataType(dataType, {"MEMBERSHIP"}) then
				if self.db.realm.news.memberships == nil then
					--Setup the memberships if it doesnt yet exist.
					self.db.realm.news.memberships = {}
				end

				--Loop through the memberships now and check which ones are newer.


				--Get a list of all the memberships
				local memberships = {}
				for _,membership in ipairs(self.db.realm.news.memberships) do
					--loop through the memberships
					local isNew = true
					for k,v in pairs(memberships) do
						if v.name == membership.name and v.data == membership.data and v.time == membership.time then
							isNew = false
							break
						end
					end
					if isNew then
						--self:Print("Adding DB "..membership.name..":"..membership.data.." to list of memberships")
						tinsert(memberships, membership)
					end
				end

				if not forceReq and data.news ~= nil and data.news.memberships ~= nil then
					--self:Print("Building list from Data")
					for _,membership in ipairs(data.news.memberships) do
						--self:Print(membership.name.." "..membership.data.." "..membership.time)
						local isNew = true
						for k,v in pairs(memberships) do
							if v.name == membership.name and v.data == membership.data and v.time == membership.time then
								
								isNew = false
								break
							end
						end
						if isNew then
							--self:Print("Adding Data "..membership.name..":"..membership.data.." to list of memberships")
							tinsert(memberships, membership)
						end
					end
				end
				
				
				self:Print("Total memberships: "..#memberships)
				for _,membership in ipairs(memberships) do
					--If this is being forced, or the achievement time is more recent than our time, or it doesnt exist, then 
					--self:Print("Name: "..membership.name)	
					local index = nil

					--Check to see if the achievement is in the database yet.
					local newEntry = true
					for i,v in ipairs(self.db.realm.news.memberships) do
						if membership.name == v.name and membership.data == v.data then
							newEntry = false
							index = i
							break
						end
					end

					if forceReq or newEntry or index ~= nil and membership.time > self.db.realm.news.memberships[index].time then
						if reqData.news == nil then
							reqData.news = {["time"] = self.db.realm.news.time}
						end

						if reqData.news.memberships == nil then
							reqData.news.memberships = {}
						end
						--self:Print("Adding Membership to request")
						--Set the time to 0 so it will force an update to be sent.
						tinsert(reqData.news.memberships, {["time"]=0, ["name"]=membership.name, ["data"]=membership.data})
					else
						if index ~= nil and self.db.realm.news.memberships[index].time ~= membership.time then
							--self:Print("Building SendData")
							--send them the new character data
							--Again, if the character data is different, but the main data wasnt, we would need to add the main data to the sendData table.
							if sendData.news == nil then
								sendData.news = {["time"]=data.news.time}
							end

							if sendData.news.memberships == nil then
								sendData.news.memberships = {}
							end
							
							--self:Print("Adding Membership to send")
							tinsert(sendData.news.memberships, membership)
						else
							--Just skip it since the records would equal eachother.
							--self:Print("Record already matches DB. Skipping ("..index..")")
						end
					end
				end
			end
		end		
		

		--We now have a sendData array with new info for the other player
		--And a reqData array which will request data to be sent back that the other player had that we didnt.
		--We need to send both arrays	

		if next(reqData) ~= nil then
			self:Print("Sending data request to "..sender)
			self:SendDataSync("REQUEST", sender, reqData)
			--self.db.char.reqData = reqData

		else
			self:Print("No new data was given. Skipping Request")
		end

		if next(sendData) ~= nil then
			self:Print("Sending newer data to "..sender)

			self:SendDataSync("UPDATE", sender, sendData)
			--self.db.char.sendData = sendData
		else
			self:Print("No new data was requested. Skipping Update")
		end
		return false
	else
		return true
	end

end

function MG:DataSyncUpdate(sender, data)
	self:Print("SAVING DATA TO DATABASE")
	if data ~= nil then
		--Check the Message of the Day
		if data.motd ~= nil and data.motd.time > self.db.realm.motd.time then
			self:Print("Updating the Message of the Day")
			self.db.realm.motd = data.motd
		end

		--Check the Mission Statement
		if data.mission ~= nil and data.mission.time > self.db.realm.mission.time then
			self:Print("Updating the Mission Statement")
			self.db.realm.mission = data.mission
		end
		
		--Check the Guild Info
		if data.info ~= nil and data.info.time > self.db.realm.info.time then
			self:Print("Updating the Guild Info")
			self.db.realm.info = data.info
		end

		--Check the characters
		if data.offlinePlayers ~= nil then
			for main, b in pairs(data.offlinePlayers) do
				if self.db.realm.offlinePlayers[main] == nil then
					--Add the main record
					self:Print("Adding Main["..main.."] to the database")
					self.db.realm.offlinePlayers[main] = data.offlinePlayers[main]
				else
					--Update the main level data...
					if data.offlinePlayers[main].rank ~= nil and data.offlinePlayers[main].time > self.db.realm.offlinePlayers[main].time then
						self.db.realm.offlinePlayers[main].rank = data.offlinePlayers[main].rank
						self.db.realm.offlinePlayers[main].joined = data.offlinePlayers[main].joined
						self.db.realm.offlinePlayers[main].time = data.offlinePlayers[main].time
					end

					if data.offlinePlayers[main].characters ~= nil then
						--Loop through all the new character data
						for name, d in pairs(b.characters) do
						
							if data.offlinePlayers[main].characters[name] ~= nil and (self.db.realm.offlinePlayers[main].characters[name] == nil or data.offlinePlayers[main].characters[name].time > self.db.realm.offlinePlayers[main].characters[name].time) then
								--Copy the character data
								self:Print("Character: "..name.."("..main..") will be updated")
								self.db.realm.offlinePlayers[main].characters[name] = data.offlinePlayers[main].characters[name]
							end

							if data.offlinePlayers[main].characters[name].public ~= nil and (self.db.realm.offlinePlayers[main].characters[name].public == nil or self.db.realm.offlinePlayers[main].characters[name].public.time == nil or data.offlinePlayers[main].characters[name].public.time > self.db.realm.offlinePlayers[main].characters[name].public.time) then
								--Copy the public data
								self:Print("Character-Public: "..name.."("..main..") will be updated")
								self.db.realm.offlinePlayers[main].characters[name].public = data.offlinePlayers[main].characters[name].public
							end

							if data.offlinePlayers[main].characters[name].officer ~= nil and (self.db.realm.offlinePlayers[main].characters[name].officer == nil or self.db.realm.offlinePlayers[main].characters[name].officer.time == nil or data.offlinePlayers[main].characters[name].officer.time > self.db.realm.offlinePlayers[main].characters[name].officer.time) then
								--Copy the officer data
								self:Print("Character-Officer: "..name.."("..main..") will be updated")
								self.db.realm.offlinePlayers[main].characters[name].officer = data.offlinePlayers[main].characters[name].officer
							end

							if data.offlinePlayers[main].characters[name].pvp ~= nil and (self.db.realm.offlinePlayers[main].characters[name].pvp == nil or self.db.realm.offlinePlayers[main].characters[name].pvp.time == nil or data.offlinePlayers[main].characters[name].pvp.time > self.db.realm.offlinePlayers[main].characters[name].pvp.time) then
								--Copy the pvp data
								self:Print("Character-PvP: "..name.."("..main..") will be updated")
								self.db.realm.offlinePlayers[main].characters[name].pvp = data.offlinePlayers[main].characters[name].pvp
							end
						end
					end					
				end
			end
		end
		
		--Check the News
		if self.db.realm.news == nil then
			self.db.realm.news = {["time"]=0, ["levels"]={}, ["achievements"]={}, ["memberships"]={}}
		end
		--if data.news ~= nil and data.news.time > self.db.realm.news.time then
		if data.news ~= nil then
			self:Print("Updating the News")
			self.db.realm.news.time = data.news.time
			
			--Achievements
			if self.db.realm.news.achievements == nil then
				self:Print("Adding Achievements Table")
				self.db.realm.news.achievements = {}
			end
			if data.news.achievements ~= nil then
				self:Print("Scanning provided Achievements")
				for a,b in ipairs(data.news.achievements) do
					local index = nil
					local isNewer = false
					for c,d in ipairs(self.db.realm.news.achievements) do
						if b.name == d.name and b.data == d.data then
							self:Print("Match: "..d.name..":"..d.data)
							index = c
							if b.time > d.time then
								self:Print("Newer TimeCode")
								isNewer = true
							end
							break
						end
					end
					if index ~= nil and isNewer then
						self.db.realm.news.achievements[index] = b
					else
						tinsert(self.db.realm.news.achievements, b)
					end
				end
			else
				self:Print("No Achievements were provided")
			end
			
			--Levels
			if self.db.realm.news.levels == nil then
				self:Print("Adding Levels Table")
				self.db.realm.news.levels = {}
			end
			if data.news.levels ~= nil then
				self:Print("Scanning provided Levels")
				for a,b in ipairs(data.news.levels) do
					local index = nil
					local isNewer = false
					for c,d in ipairs(self.db.realm.news.levels) do
						if b.name == d.name and b.data == d.data then
							self:Print("Match: "..d.name..":"..d.data)
							index = c
							if b.time > d.time then
								self:Print("Newer TimeCode")
								isNewer = true
							end
							break
						end
					end
					if index ~= nil and isNewer then
						self.db.realm.news.levels[index] = b
					else
						tinsert(self.db.realm.news.levels, b)
					end
				end
			else
				self:Print("No Levels were provided")
			end
			
			--Memberships
			if self.db.realm.news.memberships == nil then
				self:Print("Adding Memberships Table")
				self.db.realm.news.memberships = {}
			end
			if data.news.memberships ~= nil then
				self:Print("Scanning provided Memberships")
				for a,b in ipairs(data.news.memberships) do
					local index = nil
					local isNewer = false
					for c,d in ipairs(self.db.realm.news.memberships) do
						if b.name == d.name and b.data == d.data then
							self:Print("Match: "..d.name..":"..d.data)
							index = c
							if b.time > d.time then
								self:Print("Newer TimeCode")
								isNewer = true
							end
							break
						end
					end
					if index ~= nil and isNewer then
						self.db.realm.news.memberships[index] = b
					else
						tinsert(self.db.realm.news.memberships, b)
					end
				end
			else
				self:Print("No Memberships were provided")
			end
			
		end		
		

		--Now we have updated the database with the data from the sender
		--If we are actually doing a sync, put the sender on the updatedPlayers list.
		if doingSync then
			self:Print("Finished applying update from "..sender)
			self.db.realm.updatedPlayers[self:GetMainFromName(sender)] = self:GetTimeCode()
		end
	end

end

function MG:SendTest(playerName)
	if self:CanSend() then
		local teststring = format(self:CreateMsgTemplate(2), "TEST", playerName);
		if self.db.profile.filterDisabled then
			self:Print("Test Msg: "..teststring)
		end
		self:SendCommMessage(self.db.realm.baseChannel, teststring, 'WHISPER', playerName)
		return false
	end
	return true
end

function MG:OnCommReceived(prefix, msg, distribution, sender)
	--self:Print("Comm Recieved - Prefix: "..prefix)
	--self:Print("Comm Recieved - BaseChannel: "..self.db.realm.baseChannel)
	--self:Print("Comm Recieved - Distribution: "..distribution)
	--self:Print("Comm Recieved - Sender: "..sender)
	--self:Print("Comm Recieved - Message: "..msg)
	
	if(prefix == self.db.realm.baseChannel) and (distribution == "WHISPER" or distribution == "CHANNEL"or distribution == "BULK") then
		--> This area is for inviting new guild members<--

		-- Data Sync
		local first, last, mode, dataType, packedData, guildChannel, sendTime = string.find(msg, self:GetMsgFormatString("DATASYNC"))
		if(first ~= nil) then
			--self:Print("Just got the DataSync")
			--self:Print("Mode: "..mode)
			--self:Print("DataType: "..dataType)
			--self:Print("Guild Id: "..guildChannel)
			--self:Print("SendTime: "..sendTime)
			if sender ~= nil and guildChannel == self.db.realm.guildChannel then
				local main = self:GetMainFromName(sender)

				--Now, unpack the data
				local data = self:UnpackData(packedData)

				if mode == "CHECK" then
					if doingSync == true then
						--Send busy
						self:SendDataSync("BUSY", sender)
						self.db.realm.busyPlayers[main] = tonumber(sendTime)
					else
						--Send request
						self:DataSyncRequest(sender, nil, "ALL")
					end
					return

				elseif mode == "UPDATE" then
					self:DataSyncUpdate(sender, data)
					return
				
				elseif mode == "REQUEST" then
					self:DataSyncRequest(sender, data, dataType)
					return
					
				elseif mode == "BUSY" then
					self:Print(sender.." is currently busy updating. They will get back to you.")

					--The sender is also updating right now. They will get to use when ready.
					--Until then put the sender on the updated list

					self.db.realm.updatedPlayers[main] = tonumber(sendTime)
					return
				end
			end
		end

		-- Invite
		--local invitestring = format(self:CreateMsgTemplate(7), "INVITE", "(.+)", "(.+)", "(.+)", "(.+)")
		local first, last, sendTime, guildName, guildChannel, guildPassword = string.find(msg, self:GetMsgFormatString("INVITE"))
		if(first ~= nil) then
			
			--self:Print("senderTimeCode: "..sendTime)
			--self:Print("localTimeCode: "..time())
			local timeOffset = self:GetTimeCodeOffset(tonumber(sendTime))
			--self:Print("timeOffset: "..timeOffset)
			if(sender == nil) then
				return
			end
			if(guildName ~= nil and guildChannel ~= nil and guildPassword ~= nil) then
				self:ConfirmGuildInvite(sender, guildName, guildChannel, guildPassword, timeOffset)
			end
			return
		end
		

		-- Kick
		--first, last, guildChannel, sendTime, = string.find(msg, self:GetMsgFormatString("KICK"))
		local first, last, kicker, guildChannel, guildPassword, sendTime = string.find(msg, self:GetMsgFormatString("KICK"))

		if(first ~= nil) then
			self:Print("doing Kick")
			if(sender == nil) then
				return
			end
			self:Print("doing Kick again")
			if guildChannel == self.db.realm.guildChannel and guildPassword == self.db.realm.guildPassword then
				self:Print("data matches for kick")
				self:SendRemove(kicker)
			end
			return
		end
		
		
		-- Rank (Promote / Demote)
		local first, last, mode, guildChannel, sendTime = string.find(msg, self:GetMsgFormatString("RANK"))
		if(first ~= nil) then
			if(sender == nil) then
				return
			end
			if(guildChannel == self.db.realm.guildChannel) then
				if mode == "PROMOTE" then
					self:DoPromote(tonumber(sendTime))
				elseif mode == "DEMOTE" then
					self:DoDemote(tonumber(sendTime))
				end
			end
			return
		end

	end

	
	if(message == "REQ") then
		local tradelist = "";
		if (self.db.realm.tradeskillPerChar == true) then
			for k,v in pairs(self.db.char.professions) do
				if(v.link ~= nil) then
					tradelist = tradelist .. v.link .. specialChar;
				end
			end
		else
			for k,v in pairs(self.db.realm.professions) do
				if(v.link ~= nil) then
					tradelist = tradelist .. v.link .. specialChar;
				end
			end
		end
		self:SendCommMessage(self.db.realm.baseChannel, tradelist, "WHISPER", sender, "NORMAL");
	else
		local playerFrame = _G["MetaGuild_PlayerInfoFrame"];
		if (playerFrame:IsShown()) and (playerFrame.name == sender) then
			local tradeskilltext = "<html><body><p>";
			for k,v in ipairs(self:explode(specialChar, message)) do
				tradeskilltext = tradeskilltext .. v .. "<br/>";
				--_G["MetaGuild_PlayerInfoFrame_Tradeskills"]:AddMessage(v);
			end
			tradeskilltext = tradeskilltext .. "</p></body></html>";
			_G["MetaGuild_PlayerInfoFrame_Tradeskills"]:SetText(tradeskilltext);
		end
	end
end


function MG:GetMsgFormatString(formatType)
	
	if formatType == "CHARUPDATE" then
		--main, class, level, zone, status, rank, sendTime
		return format(self:CreateMsgTemplate(8), "CHARUPDATE", "(.+)", "(%a+)", "(%d%d)", "(.+)", "(%d)", "(%d)", "(%d+)")

	elseif formatType == "PVPUPDATE" then
		--ratingRbg, rating2v2, rating3v3, rating5v5, time
		return format(self:CreateMsgTemplate(6), "PVPUPDATE", "(%d+)", "(%d+)", "(%d+)", "(%d+)", "(%d+)")

	elseif formatType == "JOIN" then
		--main, class, level, rank
		return format(self:CreateMsgTemplate(5), "JOIN", "(.+)", "(%a+)", "(%d%d?)", "(.+)")

	elseif formatType == "INVITE" then
		--sendTime, guildName, guildChannel, guildPassword
		return format(self:CreateMsgTemplate(5), "INVITE", "(%d+)", "(.+)", "(.+)", "(.+)")
	
	elseif formatType == "KICK" then
		--kicker, guildChannel, guildPassword, sendTime
		return format(self:CreateMsgTemplate(5), "KICK", "(.+)", "(.+)", "(.+)", "(%d+)")

	elseif formatType == "RANK" then
		--mode, sendTime, guildName
		return format(self:CreateMsgTemplate(4), "RANK", "(%a+)", "(.+)", "(%d+)")

	elseif formatType == "TIMECODE" then
		--mode, main, rank, sendTime
		return format(self:CreateMsgTemplate(5), "TIMECODE", "(%a+)", "(.+)", "(%d)", "(%d+)")
		
	elseif formatType == "DATASYNC" then
		--mode, datatype, packeddata, channelName, lastUpdate
		return format(self:CreateMsgTemplate(6), "DATASYNC", "(%a+)", "(%a+)", "(.+)", "(.+)", "(%d+)")

	elseif formatType == "VERSION" then
		--lastUpdate, version, mode, display
		return format(self:CreateMsgTemplate(5), "VERSION", "(%d+)", "(.+)", "(%a+)","(%d)")
		
	elseif formatType == "NOTE" then
		--mode, note, name, sendTime
		return format(self:CreateMsgTemplate(5), "NOTE", "(%a+)", "(.+)", "(.+)", "(%d+)")
		
	elseif formatType == "ACHIEVEMENT" then
		--id, sendTime
		return format(self:CreateMsgTemplate(3), "ACHIEVEMENT", "(%d+)", "(%d+)")

	elseif formatType == "CHARLEVELUP" then
		--level, sendTime
		return format(self:CreateMsgTemplate(3), "CHARLEVELUP", "(%d+)", "(%d+)")
		
	elseif formatType == "TEST" then
		--lastUpdate, version, mode, display
		return format(self:CreateMsgTemplate(2), "TEST", "(.+)")
	end
	return
end


function MG:ParseMessage(player, msg)
	local ret = nil;
	
	--255 letter max message
	
	--NOTE: All numbers received in this way will be strings and needs to be converted to numbers before passing along to other functions.

	--Time Code
	local first, last, mode, main, rank, sendTime = string.find(msg, self:GetMsgFormatString("TIMECODE"))	
	if(first ~= nil) then
		if mode == "SEND" then
			local timeOffset = self:GetTimeCodeOffset(tonumber(sendTime))
			if rank == 3 then
				--This is the guild master and use this new value for your timecode Offset
				self.db.realm.timeOffset = timeOffset
			end
			return
		elseif mode == "REQUEST" then
			local timeCode = self:GetTimeCode()
			SendChatMessage(timeCode, self.db.realm.guildChannel);
			return		
		end
	end

	-- Character Update  (rough max 70 chars)
	local first, last, main, class, level, zone, status, rank, sendTime = string.find(msg, self:GetMsgFormatString("CHARUPDATE"))
	if(first ~= nil) then
		self.db.realm.onlinePlayers[player] = { ["main"] = main, ["status"] = tonumber(status) }
		self:UpdateOfflinePlayerlist(player, {["main"] = main, ["class"] = class, ["level"] = tonumber(level), ["zone"] = zone, ["time"] = tonumber(sendTime), ["rank"] = tonumber(rank)})
		return
	end


	-- Character PVP Update  (rough max 70 chars)
	local first, last, ratingBg, rating2v2, rating3v3, rating5v5, timeCode = string.find(msg, self:GetMsgFormatString("PVPUPDATE"))
	if(first ~= nil) then
		self:UpdatePlayerPvpStats(player, {["ratingBg"]=tonumber(ratingBg), ["rating2v2"]=tonumber(rating2v2), ["rating3v3"]=tonumber(rating3v3), ["rating5v5"]=tonumber(rating5v5), ["time"]=tonumber(timeCode)})
		return
	end
		

	-- A player joins
	local first, last, main, class, level, rank = string.find(msg, self:GetMsgFormatString("JOIN"));
	if(first ~= nil) then
	
		--self:Print("Doing a Character Join")
	
		if(player ~= main) then
			ret = player .. "(" .. main .. ") joined."
		else
			ret = player .. " joined."
		end

		self.db.realm.onlinePlayers[player] = { ["main"] = main, ["status"] = 1 }
		self:UpdateOfflinePlayerlist(player, {["main"] = main, ["class"] = class, ["level"] = tonumber(level), ["zone"] = "Unknown", ["time"] = self:GetTimeCode(), ["rank"] = tonumber(rank)})
		self:SendUpdate()
		return ret
	end
	
	-- A player leaves
	local first, last = string.find(msg, "LEAVE"..specialChar);
	if(first ~= nil) then
		--NOTE: I am not sure this is even being used anymore.
		self:SystemMessage(player .. " left");
		self.db.realm.onlinePlayers[player] = nil;
		self:UpdateOnlineCount()
		self:Print("Player Leaves... Message parse")
		return; --player .. " left";
	end	
	
	-- A player is being Kicked
	local first, last, kicker, guildChannel, guildPassword, sendTime = string.find(msg, self:GetMsgFormatString("KICK"));	
	if(first ~= nil) then
		local main = self:GetMainFromName(player)
		if self.db.realm.offlinePlayers[main] ~= nil and guildChannel == self.db.realm.guildChannel and guildPassword == self.db.realm.guildPassword then
			self:AddNewsEvent(player, tonumber(sendTime), 3, -100)
			self:DoRemove(main, kicker, tonumber(sendTime))
		end
		return
	end
	

	-- A note has been updated
	local first, last, mode, text, name, sendTime = string.find(msg, self:GetMsgFormatString("NOTE"))
	if(first ~= nil) then
		local author = self:GetMainFromName(player)
		local main = self:GetMainFromName(name)
		if mode == "PUBLIC" then					
			self.db.realm.offlinePlayers[main].characters[name].public = {["text"]=text, ["author"]=author, ["time"]=tonumber(sendTime)}		
		elseif mode == "OFFICER" then
			self.db.realm.offlinePlayers[main].characters[name].officer = {["text"]=text, ["author"]=author, ["time"]=tonumber(sendTime)}	
		end
		MetaGuild_PlayerinfoUpdate(name, main)
		return
	end	
	
	-- Achievement
	local first, last, achievementId, sendTime = string.find(msg, self:GetMsgFormatString("ACHIEVEMENT"))
	--first, last, link = string.find(msg, specialChar.."ACHIEVEMENT" .. specialChar .. "(.+)" .. specialChar)
	if(first ~= nil) then
		self:Print("Just got an acheivement msg")
		--insert it into the database		
		self:AddNewsEvent(player, tonumber(sendTime), 1, tonumber(achievementId))

		--self:Print("Just had an Achievement show up")
		if(self.db.profile.showAchiev) then
			self:Print("Printing Achievement")
			local link = self:removeLinks(GetAchievementLink(achievementId))
			--self:Print(format(L["AchievementEarned"], player, self:addLinks(link)))
			return format(L["AchievementEarned"], player, self:addLinks(link))
		end
	end
	

	-- Level Up
	local first, last, level, sendTime = string.find(msg, self:GetMsgFormatString("CHARLEVELUP"))
	if(first ~= nil) then		
		--insert it into the database
		--self:Print("Level Up Recieved")
		self:AddNewsEvent(player, tonumber(sendTime), 2, tonumber(level))
		--Now send the update	
		self:SendUpdate()
		if(self.db.profile.showLevel) then
			return format(L["LevelEarned"], player, level)
		end
			
	end


	-- Version Check
	local first, last, sendTime, version, mode, display = string.find(msg, self:GetMsgFormatString("VERSION"));	
	if(first ~= nil) then
		if mode == "Request" then
			--Request
			SendChatMessage(("Version : "..self.db.realm.version), self.db.realm.guildChannel)
			self:SendVersion()
		elseif mode == "Display" then
			--Report
			--SendChatMessage(("Version : "..version), self.db.realm.guildChannel)

			--Now lets check to see if this is a newer version than the one I have.
			if self:IsNewVersion(version) then
				self:Print("There is a new version avalible")
				if displayNewVersion then
					self:DisplayNewVersionWarning()
				end
			end
		end
		return
	end
end




function MG:GetCharacterInfo(name)	
	local main = self:GetMainFromName(name)

	if self.db.realm.offlinePlayers[main] ~= nil and self.db.realm.offlinePlayers[main].characters ~= nil then
		--Quick fix to make sure the main has a joined date. Once everyone has been updated, it will no longer be needed.
		if self.db.realm.offlinePlayers[main].joined == nil then
			self.db.realm.offlinePlayers[main].joined = self.db.realm.offlinePlayers[main].time
		end

		--Check to make sure they are not placeholders. Placeholders all have a timecode of 0.
		if tonumber(self.db.realm.offlinePlayers[main].characters[name].time) > 0 then

			local character = {}

			for k,v in pairs(self.db.realm.offlinePlayers[main].characters[name]) do
				--Loop through all the data and make a copy of that.
				if type(v) ~= "table" then
					character[k] = v
				end	
			end

			if self.db.realm.offlinePlayers[main].characters[name].pvp ~= nil then
				--This is a quick fix for the lack of timecodes in an earlier version. Once everyone has been updated, it will no longer be needed.
				if self.db.realm.offlinePlayers[main].characters[name].pvp.time == nil then
					self.db.realm.offlinePlayers[main].characters[name].pvp.time = self:GetTimeCode()
				end

				character.rating2v2 = self.db.realm.offlinePlayers[main].characters[name].pvp.rating2v2
				character.rating3v3 = self.db.realm.offlinePlayers[main].characters[name].pvp.rating3v3
				character.rating5v5 = self.db.realm.offlinePlayers[main].characters[name].pvp.rating5v5
				character.ratingBg  = self.db.realm.offlinePlayers[main].characters[name].pvp.ratingBg
			else
				character.rating2v2 = "???"
				character.rating3v3 = "???"
				character.rating5v5 = "???"
				character.ratingBg  = "???"
			end
			character.name = name
			character.main = main
			character.rank = tonumber(self.db.realm.offlinePlayers[main].rank)
			
			if self.db.realm.offlinePlayers[main].characters[name].public == nil then
				self.db.realm.offlinePlayers[main].characters[name].public = {["time"]=0}
			end
			
			if self.db.realm.offlinePlayers[main].characters[name].officer == nil then
				self.db.realm.offlinePlayers[main].characters[name].officer = {["time"]=0}
			end
			
			character.public = self.db.realm.offlinePlayers[main].characters[name].public
			character.officer = self.db.realm.offlinePlayers[main].characters[name].officer
			
			if self.db.realm.onlinePlayers[name] ~= nil then
				character.status = self.db.realm.onlinePlayers[name].status
			else
				character.status = 0
			end

			return character
		end
	end
	--self:Print("No character named "..name.." could be found in the database.")
	return nil
end

function MG:CharacterIsOnline(characterName)
	for k, v in pairs(self.db.realm.onlinePlayers) do
		if characterName == k then
			return true
		end
	end
	return false
end


function MG:UpdatePlayerPvpStats(name, data)
	--self:Print("Updating PvP stats")
	local main = self:GetMainFromName(name)
	if main ~= nil then
		if self.db.realm.offlinePlayers[main] ~= nil and self.db.realm.offlinePlayers[main].characters ~= nil and self.db.realm.offlinePlayers[main].characters[name] ~= nil then
			self.db.realm.offlinePlayers[main].characters[name].pvp = {["ratingBg"]=data.ratingBg, ["rating2v2"]=data.rating2v2, ["rating3v3"]=data.rating3v3, ["rating5v5"]=data.rating5v5, ["time"]=data.time}
		end
	end
	MetaGuildRoster_Update()
end

function MG:UpdateOfflinePlayerlist(name, data)
	local main
	if data["main"] ~= nil then
		main = data["main"]
	end

	--We need to have a main selected at this point. Without it, we cannot sort the data.
	if main ~= nil then		
		--Lets see if the main has already been saved to the db.
		if self.db.realm.offlinePlayers[main] == nil then		
			--The main does not exist. This could be because its a new record, or that the user has switched mains.
			--Lets loop through the offline characters and see if the new main exists already. If it does, the user switched mains.
			--And we will need to update the database to reflect that.
			local oldMain = nil
			for a, b in pairs(self.db.realm.offlinePlayers) do --"a" is the main, "b" is the list of characters
				for c, d in pairs(b.characters) do --"c" is the character name, "d" is the attributes
					if c == main then
						oldMain = a
						--We found the main in the database already.
						--Set oldMain so it will update the database to reflect it as the main
					end
				end
			end
			
			if oldMain == nil then
				self:Print("Adding new main record for ".. main)
				self.db.realm.offlinePlayers[main] = {["characters"] = {}, ["rank"]=0, ["time"]=0}
			else
				self.db.realm.offlinePlayers[main] = self.db.realm.offlinePlayers[oldMain]  --assign the old record to the new main key
				self.db.realm.offlinePlayers[oldMain] = nil --Remove the old main key				
			end
		end

		if self.db.realm.offlinePlayers[main] == nil then
			self:Print("Main ("..main..") does not exist in the data base for some reason. This should not happen at this point.")
		end


		--The main either existed or has been created. 
		--Now lets see if the current character has been placed in the database.		
		if self.db.realm.offlinePlayers[main].characters[name] == nil then   
			--The character does not exist yet in the character array. We will create it so the next part can add the data to it.
			self.db.realm.offlinePlayers[main].characters[name] = {["class"]="", ["level"]="", ["zone"]="", ["time"]=0, ["public"]={["time"]=0}, ["officer"]={["time"]=0}, ["pvp"]={["time"]=0}}
		end
		
		if self.db.realm.offlinePlayers[main].characters[main] == nil then
			self.db.realm.offlinePlayers[main].characters[main] = {["class"]="???", ["level"]="??", ["zone"]="???", ["time"]=0, ["public"]={["time"]=0}, ["officer"]={["time"]=0}, ["pvp"]={["time"]=0}}
		end

		--The character has already been assigned to the database. Now all that is left is to update it.
		for k,v in pairs(data) do --loop through all the new character data. (k,v) == (attr, data)
			--self:Print(k)
			if k == "time" and v == nil then
				v = self:GetTimeCode()
			end
			if k == "rank" then
				self.db.realm.offlinePlayers[main].rank = v
			elseif k == "main" then
				--Do nothing, we do not want to save the main to the character array
			else
				self.db.realm.offlinePlayers[main].characters[name][k] = v
			end
			
			if k == "time" then
				self.db.realm.offlinePlayers[main].time = v
			end
		end
	end
	self:UpdateOnlineCount()
end

function MG:RepairDatabase()
	local isInvalid
	local clearCharacterData = {"bgrating", "arenarating", "ratingBg", "bgRating", "arenarating2v2", "rating2v2", "arenarating3v3", "rating3v3", "arenarating5v5", "rating5v5", "name", "main"}
	local pvpAttrs = {"ratingBg", "rating2v2", "rating3v3", "rating5v5", "time"}
	for main, b in pairs(self.db.realm.offlinePlayers) do
		isInvalid = true
		for x,y in pairs(b) do
			if x == "characters" and y ~= nil then
				for name,d in pairs(y) do
					self:Print("Checking "..name)
					if name == main then
						isInvalid = false
					end

					for e,f in pairs(d) do
						--self:Print(e)
						for g,h in ipairs(clearCharacterData) do
							if h == e then
								self:Print("Removing "..h)
								self.db.realm.offlinePlayers[main].characters[name][h] = nil
							end
						end

						if e == "time" or e == "level" then
							self.db.realm.offlinePlayers[main].characters[name][e] = tonumber(self.db.realm.offlinePlayers[main].characters[name][e])

						elseif e == "pvp" and self.db.realm.offlinePlayers[main].characters[name][e].ratingBg ~= nil then
							for i, j in ipairs(pvpAttrs) do
								--self:Print(j)
								if self.db.realm.offlinePlayers[main].characters[name][e][j] == nil then
									self.db.realm.offlinePlayers[main].characters[name][e][j] = 0
									--self:Print("fixing "..j)
								end
								
								self.db.realm.offlinePlayers[main].characters[name][e][j] = tonumber(self.db.realm.offlinePlayers[main].characters[name][e][j])
							end
							--if self.db.realm.offlinePlayers[main].characters[name][e].ratingBg == nil then
							--	self.db.realm.offlinePlayers[main].characters[name][e].ratingBg

							--self.db.realm.offlinePlayers[main].characters[name][e].ratingBg = tonumber(self.db.realm.offlinePlayers[main].characters[name][e].ratingBg)
							--self.db.realm.offlinePlayers[main].characters[name][e].rating2v2 = tonumber(self.db.realm.offlinePlayers[main].characters[name][e].rating2v2)
							--self.db.realm.offlinePlayers[main].characters[name][e].rating3v3 = tonumber(self.db.realm.offlinePlayers[main].characters[name][e].rating3v3)
							--self.db.realm.offlinePlayers[main].characters[name][e].rating5v5 = tonumber(self.db.realm.offlinePlayers[main].characters[name][e].rating5v5)
							--self.db.realm.offlinePlayers[main].characters[name][e].time = tonumber(self.db.realm.offlinePlayers[main].characters[name][e].time)
						elseif e == "public" or e == "officer" then
							self.db.realm.offlinePlayers[main].characters[name][e].time = tonumber(self.db.realm.offlinePlayers[main].characters[name][e].time)
						end
					end
				end
			elseif x == "time" or x == "rank" then
				self.db.realm.offlinePlayers[main][x] = tonumber(self.db.realm.offlinePlayers[main][x])
			end
		end
		if isInvalid then
			self.db.realm.offlinePlayers[main] = nil
		end
	end
	self:UpdateOnlineCount()
	
	if self.db.realm.mission.time ~= nil then
		self.db.realm.mission.time = tonumber(self.db.realm.mission.time)
	end
	if self.db.realm.motd.time ~= nil then
		self.db.realm.motd.time = tonumber(self.db.realm.motd.time)
	end
	
	
	--This isnt part of the offlinePlayers table, but should be cleaned up anyway just to help reduce the size of the saved variables.
	if self.db.char.reqData ~= nil then
		self.db.char.reqData = nil
	end
	if self.db.char.sendData ~= nil then
		self.db.char.sendData = nil
	end
	
	if self.db.realm.achievements ~= nil then
		self.db.realm.achievements = nil
	end
	
	--Repair the news
	if self.db.realm.news == nil then
		self.db.realm.news = {["time"]=0, ["levels"]={}, ["achievements"]={}, ["memberships"]={}}
	end
	
	--Repair Achievements
	if self.db.realm.news.achievements == nil then
		self.db.realm.news.achievements = {}
	else
		local achievements = {}
		for a,b in ipairs(self.db.realm.news.achievements) do
			if b.name ~= nil and b.data ~= nil then
				if b.time == nil then
					b.time = 0
				end
				
				--remove duplicates
				local isUnique = true
				for c,d in ipairs(achievements) do
					if b.name == d.name and b.data == d.data then
						isUnique = false		
					end
				end
				if isUnique then
					tinsert(achievements, b)
				end
			end
		end
		self.db.realm.news.achievements = achievements
	end
	
	--Repair Levels
	if self.db.realm.news.levels == nil then
		self.db.realm.news.levels = {}
	else
		local levels = {}
		for a,b in ipairs(self.db.realm.news.levels) do
			if b.name ~= nil and b.data ~= nil then
				if b.time == nil then
					b.time = 0
				end
				
				--remove duplicates
				local isUnique = true
				for c,d in ipairs(levels) do
					if b.name == d.name and b.data == d.data then
						isUnique = false		
					end
				end
				if isUnique then
					tinsert(levels, b)
				end
			end
		end
		self.db.realm.news.levels = levels
	end
	
	--Repair Membership
	if self.db.realm.news.memberships == nil then
		self.db.realm.news.levels = {}
	else
		local memberships = {}
		for a,b in ipairs(self.db.realm.news.memberships) do
			if b.name ~= nil and b.data ~= nil then
				if b.time == nil then
					b.time = 0
				end
				
				--remove duplicates
				local isUnique = true
				for c,d in ipairs(memberships) do
					if b.name == d.name and b.data == d.data then
						isUnique = false		
					end
				end
				if isUnique then
					tinsert(memberships, b)
				end
			end
		end
		self.db.realm.news.memberships = memberships
	end
	
	
	self:Print("Database Repair complete")
end

function  MG:DisplayPlayerList(playerList, isOffline)
	for k, v in pairs(playerList) do
		if v.main == "" then
			v.main = "ERROR"
		end

		local charactersStr = ""
		if isOffline then
			charactersStr = ""
			for y,z in pairs(v.characters) do
				if z.time > 0 then  --This will ignore all placeholders
					if charactersStr == "" then
						charactersStr = (charactersStr..y.."-"..z.level.."-"..z.class)
					else
						charactersStr = (charactersStr..", "..y.."-"..z.level.."-"..z.class)
					end
				end
			end
			self:Print("Main: "..k..", Characters: ["..charactersStr.."]")
		else
			self:Print("Name: "..k..", Main:"..v.main..", Status: "..self:GetStatusText(v.status))
			--self:Print("Name: "..k..", Main:"..v.main)

		end
	end
end

function MG:UpdateOnlineCount()
	local i = 0
	local j = 0
	
	for k, v in pairs(self.db.realm.onlinePlayers) do
		i = i+1
	end
	
	for k, v in pairs(self.db.realm.offlinePlayers) do
		j = j+1
	end
	numOnline = i
	dataobj.text = "MG (" .. numOnline .. ")"
	MetaGuildFrameMembersCount:SetText(format(L["Main/MembersOnlineCount"], i, j))
	MetaGuildRoster_Update()
	self:IsGuildedUpdate()
end

function MG:CanInvite(playerName)
	local name = UnitName("player")
	local rank = self:GetRank(name)
	if rank ~= nil and strlower(playerName) ~= strlower(name) and rank >= 2 then
		return true
	end
	return false
end


function MG:CanKick(playerName)
	local name = UnitName("player")
	local rank = self:GetRank(name)
	local targetRank = self:GetRank(playerName)
	if targetRank ~= nil and rank > targetRank and strlower(playerName) ~= strlower(name) then
		return true
	else
		return false
	end

end


--> Send Functions <--

function MG:SendInvite(playerName)
	if self:CanSend() and self:CanInvite(playerName) then
		local invitestring = format(self:CreateMsgTemplate(5), "INVITE", self:GetTimeCode(), self.db.realm.guildName, self.db.realm.guildChannel, self.db.realm.guildPassword);
		if self.db.profile.filterDisabled then
			self:Print("Invite Msg: "..invitestring)
		end
		self:SendCommMessage(self.db.realm.baseChannel, invitestring, 'WHISPER', playerName)
		return false
	end
	return true
end

function MG:SendKick(playerName)
	if (self:CanSend() and self:CanKick(playerName)) then
		local kickstring = format(self:CreateMsgTemplate(5), "KICK", self:GetMainFromName(UnitName("player")), self.db.realm.guildChannel, self.db.realm.guildPassword, self:GetTimeCode())
		self:SendCommMessage(self.db.realm.baseChannel, kickstring, 'WHISPER', playerName)
		return false
	end
	return true
end

function MG:SendPromote(playerName)
	if (self:CanSend()) then
		local rankString = format(self:CreateMsgTemplate(4), "RANK", "PROMOTE", self.db.realm.guildChannel, self:GetTimeCode())
		self:SendCommMessage(self.db.realm.baseChannel, rankString, 'WHISPER', playerName)
		return false
	else
		return true
	end
end

function MG:SendDemote(playerName)
	if (self:CanSend()) then
		local rankString = format(self:CreateMsgTemplate(4), "RANK", "DEMOTE", self.db.realm.guildChannel, self:GetTimeCode());
		self:SendCommMessage(self.db.realm.baseChannel, rankString, 'WHISPER', playerName)
		return false
	else
		return true
	end
end

function MG:SendUpdate()
	if (self:CanSend()) then
		local updateString = format(self:CreateMsgTemplate(8), "CHARUPDATE", self.db.realm.main, select(2,UnitClass("player")), UnitLevel("player"), self:GetZone(), status, self.db.realm.rank, self:GetTimeCode());
		SendChatMessage(updateString, self.db.realm.guildChannel)
		return false
	else
		return true
	end
end


function MG:SendLevelUp(level)
	if (self:CanSend()) then
		--self:Print("Sending Level Up")
		local levelUpString = format(self:CreateMsgTemplate(3), "CHARLEVELUP", level, self:GetTimeCode())
		SendChatMessage(levelUpString, self.db.realm.guildChannel)
		return false
	else
		return true
	end
end

function MG:SendNote(mode, author, name, note)
	if (self:CanSend()) then
		--mode, note, name, sendTime
		local noteString = format(self:CreateMsgTemplate(5), "NOTE", mode, note, name, self:GetTimeCode())
		SendChatMessage(noteString, self.db.realm.guildChannel)
		return false
	else
		return true
	end
end

function MG:SendTimeCode()
	if (self:CanSend()) then
		local timecodestring = format(self:CreateMsgTemplate(5), "TIMECODE", "SEND", self.db.realm.main, self.db.realm.rank, self:GetTimeCode());
		SendChatMessage(timecodestring, self.db.realm.guildChannel);
	end
end

function MG:RequestTimeCode()
	if (self:CanSend()) then
		local timecodestring = format(self:CreateMsgTemplate(5), "TIMECODE", "REQUEST", self.db.realm.main, self.db.realm.rank, self:GetTimeCode());
		SendChatMessage(timecodestring, self.db.realm.guildChannel);
	end
end

function MetaGuildSendJoin()
	self:SendJoin()
end

function MG:SendJoin()
	if (self:CanSend()) then
		--self:Print("Main: "..self.db.realm.main)
		--self:Print("Character: "..UnitName("player"))
		--self:Print("Class: "..select(2,UnitClass("player")))
		--self:Print("Level: "..UnitLevel("player"))
		--self:Print("Rank: "..self.db.realm.rank)

		local joinstring = format(self:CreateMsgTemplate(5), "JOIN", self.db.realm.main, select(2,UnitClass("player")), UnitLevel("player"), self.db.realm.rank)
		SendChatMessage(joinstring, self.db.realm.guildChannel)
		return false
	else
		self:Print("Cannot send join message.")
		return true
	end
end



function MG:SendAchievement(id)
	if (self:CanSend()) then
		self:Print("Sending Achiv")
		local achievementString = format(self:CreateMsgTemplate(3), "ACHIEVEMENT", id, self:GetTimeCode())
		SendChatMessage(achievementString, self.db.realm.guildChannel)
		return false
	else
		self:Print("Cannot send achievement message.")
		return true
	end
end

function MG:SendPvpUpdate(force)
	if (self:CanSend()) then
		local personalBGRating, ratedBGreward,allPointsThisWeek,allMaxPointsThisWeek,rbgPointsThisWeek,rbgMaxPointsThisWeek, seasonWins, seasonPlayed, weeklyWins, weeklyPlayed = GetPersonalRatedBGInfo()

		--Time to Get the Arena info
		local arenaInfo = {}
		for index=1, 3 do	
			ArenaTeamRoster(index)
			local teamName, teamSize, teamRating, weekPlayed, weekWins, seasonPlayed, seasonWins, playerPlayed, seasonPlayerPlayed, teamRank, playerRating = GetArenaTeam(index)
			if teamSize ~= nil then
				arenaInfo[teamSize] = {["name"]=teamName, ["rating"]=teamRating, ["playerRating"] = playerRating, ["teamRank"]=teamRank}
			end
		end
		
		--Some of the data might be nil if that bracket isnt on a team.
		local rating2v2, rating3v3, rating5v5, ratingBg
		
		if personalBGRating == nil then
			ratingBg = 0
		else
			ratingBg = personalBGRating
		end
		
		if arenaInfo[2] == nil then
			rating2v2 = 0
		else
			rating2v2 = arenaInfo[2].playerRating
		end
		
		if arenaInfo[3] == nil then
			rating3v3 = 0
		else
			rating3v3 = arenaInfo[3].playerRating
		end
		
		if arenaInfo[5] == nil then
			rating5v5 = 0
		else
			rating5v5 = arenaInfo[5].playerRating
		end
		
		
		local name = UnitName("player")
		local main = self:GetMainFromName(name)
		if self.db.realm.offlinePlayers ~= nil and self.db.realm.offlinePlayers[main] ~= nil and self.db.realm.offlinePlayers[main].characters ~= nil and self.db.realm.offlinePlayers[main].characters[name] ~= nil and self.db.realm.offlinePlayers[main].characters[name].pvp ~= nil then
			local pvp = self.db.realm.offlinePlayers[main].characters[name].pvp
			force = tonumber(force)
			if force == 1 or tonumber(ratingBg) ~= tonumber(pvp.ratingBg) or tonumber(rating2v2) ~= tonumber(pvp.rating2v2) or tonumber(rating3v3) ~= tonumber(pvp.rating3v3) or tonumber(rating5v5) ~= tonumber(pvp.rating5v5) then

				--self:Print(ratingBg.." == "..self.db.realm.offlinePlayers[main].characters[name].pvp.ratingBg)
				--self:Print(rating2v2.." == "..self.db.realm.offlinePlayers[main].characters[name].pvp.rating2v2)
				--self:Print(rating3v3.." == "..self.db.realm.offlinePlayers[main].characters[name].pvp.rating3v3)
				--self:Print(rating5v5.." == "..self.db.realm.offlinePlayers[main].characters[name].pvp.rating5v5)

				local pvpUpdateString = format(self:CreateMsgTemplate(6), "PVPUPDATE", ratingBg, rating2v2, rating3v3, rating5v5, self:GetTimeCode());
				SendChatMessage(pvpUpdateString, self.db.realm.guildChannel)
				--self:Print("Sending PvP data")
				return false

			else
				--self:Print("PvP data is the same. Will not send.")
				return true			
			end
		end
		

	else
		self:Print("Cannot send join message.")
		return true
	end
end

function MG:SendRemove(kicker)
	if (self:CanSend() and kicker ~= nil) then
		self:Print("got into Send Remove")
		local removestring = format(self:CreateMsgTemplate(5), "KICK", kicker, self.db.realm.guildChannel, self.db.realm.guildPassword, self:GetTimeCode())	
		SendChatMessage(removestring, self.db.realm.guildChannel)
		return false
	else
		return true
	end
end

function MG:SendVersion()
	if (self:CanSend()) then
		local mode = "Display"
		local versionString = format(self:CreateMsgTemplate(4), "VERSION", self:GetTimeCode(), self.db.realm.version, mode)
		SendChatMessage(versionString, self.db.realm.guildChannel)
		return false
	else
		return true
	end
end

function MG:SendVersionRequest(doDisplay)
	if doDisplay == nil then
		doDisplay = "0"
	end
	if (self:CanSend()) then
		local mode = "Request"
		local versionReqString = format(self:CreateMsgTemplate(5), "VERSION", self:GetTimeCode(), self.db.realm.version, mode, doDisplay);
		SendChatMessage(versionReqString, self.db.realm.guildChannel)
		return false
	else
		return true
	end
end

function MG:IsNewVersion(versionA)
	local valueA = self:GetVersionValue(versionA)
	local valueB = self:GetVersionValue(self.db.realm.version)	
	if valueA > valueB then
		return true
	else
		return false
	end
end

function MG:GetVersionValue(version)
	local major, minor, sub = strsplit(".", version, 3)
	return (tonumber(major)*100)+(tonumber(minor)*10)+(tonumber(sub))
end

function MG:DisplayNewVersionWarning()
	StaticPopupDialogs["NEW_METAGUILD_VERSION"] = {
		text =  L["Info/DisplayNewVersionText"],
		button1 = "Ok",
		OnAccept = function(widget)
			displayNewVersion = false
			widget:Hide()
		end,
		sound = "levelup2",
		timeout = 30,
		whileDead = true,
		hideOnEscape = true,
	}
	StaticPopup_Show ("NEW_METAGUILD_VERSION")
end

function MG:StartSyncDatabase()
	--Clear the tables
	self.db.realm.busyPlayers = {}
	self.db.realm.updatedPlayers = {}

	--Clean your database first
	self:RepairDatabase()
	
	--Put yourself on the update list since you really dont need to update with yourself.
	self.db.realm.updatedPlayers[self.db.realm.main] = self:GetTimeCode()
	
	if self:IsInGuild() then
		if self:CanSend() then
			self:Print("Starting Database Sync")
			doingSync = true
			self.syncTimerCount = 0
			self.syncTimer = self:ScheduleRepeatingTimer("SyncDatabase", self.db.realm.syncDelay)
		else
			self:Print("Cannot communicate. Sync cancelled")
			self.syncTimerCount = nil
			doingSync = false
		end
	end
end

function MG:SyncDatabase()
	self.syncTimerCount = self.syncTimerCount + 1
	if self:HasPlayersToSync() and self.syncTimerCount < 20 and self:CanSend() then
		local name = self:GetPlayerToSync()
		self:Print("Trying to Sync with "..name)
		self:SyncDatabaseWithPlayer(name)
	else
		self:Print("Sync Complete")
		self:CancelTimer(self.syncTimer)
		doingSync = false
	end
end

function MG:HasPlayersToSync()
	for k, v in pairs(self.db.realm.onlinePlayers) do
		if self.db.realm.updatedPlayers[v.main] == nil then
			self:Print("There are still players to Sync with")
			return true
		end
	end
	return false
end

function MG:GetPlayerToSync()
	--Check all online players
	for k, v in pairs(self.db.realm.onlinePlayers) do
		--If they have not been updated yet and are not busy
		--self:Print("Main?: "..v.main)
		if self.db.realm.updatedPlayers[v.main] == nil and self.db.realm.busyPlayers[v.main] == nil then
			self:Print("Selecting "..k.." to sync with")
			return k
		end
	end
	
	--Now that we have gone through all the avalible members on the list who were not busy,
	--Lets try those on the busy list to see if they are avalible yet.
	for k, v in pairs(self.db.realm.busyPlayers) do
		if self.db.realm.onlinePlayers[k] ~= nil then
			self:Print("Selecting "..k.." off the busy list to sync with")
			return k
		end
	end
	return
end

function MG:SyncDatabaseWithPlayer(name)
	local main = self:GetMainFromName(name)
	self:Print("Sending Sync Request to "..name)
	
	--send it to the player
	--Init the sync
	--local result = self:DataSyncRequest(name, nil, "ALL")
	local result = self:SendDataSync("CHECK", name)
	
	if (result) then
		self:CancelTimer(self.syncTimer)
		doingSync = false
		self:Print("Sync Cancelled")
	else
		self.db.realm.updatedPlayers[main] = self:GetTimeCode()
	end
end

function MG:SendDataSync(mode, targetName, data, dataType)
	if (self:CanSend()) then
		if mode ~= "REQUEST" and mode ~= "UPDATE" and mode ~= "CHECK" and mode ~= "BUSY" then
			self:Print("Invalid mode supplied to SendDataSync")
			return
		end
		
		if dataType == nil then
			dataType = "ALL"
		end
		
		if data == nil then
			--Make an empty array.
			data = {}
		end
		
		local priority = "NORMAL"
		if mode == "REQUEST" or mode == "UPDATE" then
			priority = "BULK"
		end
		
		local packedData = self:PackData(data)
		local dataSyncString = format(self:CreateMsgTemplate(6), "DATASYNC", mode, dataType, packedData, self.db.realm.guildChannel, self:GetTimeCode())
		self:SendCommMessage(string.upper(self.db.realm.baseChannel), dataSyncString, 'WHISPER', targetName, priority)
		
		if mode == "REQUEST" then
			self:Print("Sent data request to "..targetName)
		elseif mode == "UPDATE" then
			self:Print("Sent data update to "..targetName)
		elseif mode == "CHECK" then
			self:Print("Sent data check to "..targetName)
		elseif mode == "BUSY" then
			self:Print("Sent data busy to "..targetName)

		end
		return false
	else
		return true
	end
end

function MG:resolveSpecial(msg)
	msg = string.gsub(msg, format(colorChar, 'r'), red);
	msg = string.gsub(msg, format(colorChar, 'g'), green);
	msg = string.gsub(msg, format(colorChar, 'b'), blue);
	msg = string.gsub(msg, format(colorChar, 'n'), "\124r");
	
	return msg;
end

function MG:removeLinks(msg)
	msg = string.gsub(msg, "|c", specialLinkChar.."c");
	msg = string.gsub(msg, "|Hitem:", specialLinkChar.."Hitem:");
    msg = string.gsub(msg, "|Henchant:", specialLinkChar.."Henchant:");
	msg = string.gsub(msg, "|Hspell:", specialLinkChar.."Hspell:");
	msg = string.gsub(msg, "|Hquest:", specialLinkChar.."Hquest:");
	msg = string.gsub(msg, "|Htrade", specialLinkChar.."Htrade");
	msg = string.gsub(msg, "|Hglyph", specialLinkChar.."Hglyph");
	msg = string.gsub(msg, "|Htalent", specialLinkChar.."Htalent");
	msg = string.gsub(msg, "|Hachievement", specialLinkChar.."Hachievement");
	msg = string.gsub(msg, "|Hinstancelock", specialLinkChar.."Hinstancelock");
    msg = string.gsub(msg, "|h|r", specialLinkChar.."h"..specialLinkChar.."r");
    msg = string.gsub(msg, "|h", specialLinkChar.."h");
	msg = string.gsub(msg, "|r", specialLinkChar.."r");
	msg = string.gsub(msg, "|T", specialLinkChar.."T");
	msg = string.gsub(msg, "|t", specialLinkChar.."t");
	
	return msg;
end

function MG:addLinks(msg)
	msg = string.gsub(msg, specialLinkChar.."c", "|c");
	msg = string.gsub(msg, specialLinkChar.."Hitem:", "|Hitem:");
    msg = string.gsub(msg, specialLinkChar.."Henchant:", "|Henchant:");
	msg = string.gsub(msg, specialLinkChar.."Hspell:", "|Hspell:");
	msg = string.gsub(msg, specialLinkChar.."Hquest:", "|Hquest:");
	msg = string.gsub(msg, specialLinkChar.."Htrade:", "|Htrade:");
	msg = string.gsub(msg, specialLinkChar.."Hglyph:", "|Hglyph:");
	msg = string.gsub(msg, specialLinkChar.."Htalent:", "|Htalent:");
	msg = string.gsub(msg, specialLinkChar.."Hachievement:", "|Hachievement:");
	msg = string.gsub(msg, specialLinkChar.."Hinstancelock:", "|Hinstancelock:");
    msg = string.gsub(msg, specialLinkChar.."h"..specialLinkChar.."r", "|h|r");
    msg = string.gsub(msg, specialLinkChar.."h", "|h");
	msg = string.gsub(msg, specialLinkChar.."r", "|r");
	msg = string.gsub(msg, specialLinkChar.."T", "|T");
	msg = string.gsub(msg, specialLinkChar.."t", "|t");
	
	return msg;
end

function MetaGuild_Toggle()
	if (MetaGuildFrame:IsShown()) then
		MetaGuildFrame:Hide();
	else
		--self:InitLocalization();
		MetaGuildFrame:Show();
	end 
end


function MetaGuildFrame_TabClicked(self)
	local tabIndex = self:GetID();
	if (not self:GetParent().numTabs) then
		self:GetParent().numTabs = 4
	end
	
	PanelTemplates_SetTab(MetaGuildFrame, tabIndex);
	--print(self:GetText().." ("..tabIndex..")")
	--print(self:GetParent().numTabs)
	if (tabIndex == 1) then
		MetaGuildFrameGuildTabFrame:Show()
		MetaGuildFrameRosterTabFrame:Hide()
		MetaGuildFrameNewsTabFrame:Hide()
		MetaGuildFrameInfoTabFrame:Hide()
		MetaGuild_PlayerinfoShow("", true)
	elseif (tabIndex == 2) then
		MetaGuildFrameGuildTabFrame:Hide()
		MetaGuildFrameRosterTabFrame:Show()
		MetaGuildFrameNewsTabFrame:Hide()
		MetaGuildFrameInfoTabFrame:Hide()
	elseif (tabIndex == 3) then
		MetaGuildFrameGuildTabFrame:Hide()
		MetaGuildFrameRosterTabFrame:Hide()
		MetaGuildFrameNewsTabFrame:Show()
		MetaGuildFrameInfoTabFrame:Hide()
		MetaGuild_PlayerinfoShow("", true)
	elseif (tabIndex == 4) then
		MetaGuildFrameGuildTabFrame:Hide()
		MetaGuildFrameRosterTabFrame:Hide()
		MetaGuildFrameNewsTabFrame:Hide()
		MetaGuildFrameInfoTabFrame:Show()
		MetaGuild_PlayerinfoShow("", true)
	end
	
	--Close the edit window if its visible.
	MG:MetaGuildTextEditFrame_Hide()
	
	MG:UpdateOnlineCount()
end

function MG:IsInGuild()
	if self.db.realm.guildChannel == "" and self.db.realm.guildPassword == "" then
		return false
	else
		return true
	end
end

function MG:IsGuildedUpdate()
	if self:IsInGuild() then
		MetaGuildFrameCreateFrame:Hide()
			--MetaGuildFrameCreateFrameGuildCreateLabel:Hide()
			--MetaGuildFrameCreateFrameGuildCreateText:Hide()
			--MetaGuildFrameCreateFrameGuildCreateButton:Hide()
	else
		--Hide the tabs
		MetaGuildFrameTab1:Hide()
		MetaGuildFrameTab2:Hide()
		MetaGuildFrameTab3:Hide()
		MetaGuildFrameTab4:Hide()
		
		--And their frames
		MetaGuildFrameGuildTabFrame:Hide()
		MetaGuildFrameRosterTabFrame:Hide()
		MetaGuildFrameNewsTabFrame:Hide()
		MetaGuildFrameInfoTabFrame:Hide()
		
		MetaGuildFrameCreateFrame:Show()
		--MetaGuildFrameCreateFrameGuildCreateButton:Show()
	end
end

function MG:GetTradeskills(name)
	self:SendCommMessage("MetaGuild", "REQ", "WHISPER", name, "NORMAL");
end

function MG:explode(div,str)
  if (div=='') then return false end
  local pos,arr = 0,{}
  -- for each divider found
  for st,sp in function() return string.find(str,div,pos,true) end do
    table.insert(arr,string.sub(str,pos,st-1)) -- Attach chars left of current divider
    pos = sp + 1 -- Jump past current divider
  end
  table.insert(arr,string.sub(str,pos)) -- Attach chars right of last divider
  return arr
end

-- Profile changes
function MG:LoadedNewProfile()
	if(self.db.profile.enabled) then
		self:OnEnable();
		self.joinTimer = self:ScheduleRepeatingTimer("JoinMetaGuildChannels", 2);
	end
end

function MG:OnProfileReset()
	self:OnDisable();
end

function MG:LeaveProfile()
	if(self.db.profile.enabled) then
		-- Disable, but set to enabled without enabling
		self.db.profile.enabled = false;
		self:OnDisable();
		self.db.profile.enabled = true;
	end
end

function MetaGuildInvite(playerName)
	if self.db.realm.rank >= 2 then
		--self:Print('Invited '.. playerName)
		self:SendInvite(playerName)
	end
end

function MetaGuildKick(playerName)
	--self:Print('About to Kick '.. playerName)
	local main = self:GetMainFromName(playerName)	
	if main ~= nil and self.db.realm.offlinePlayers[main] ~= nil then
		--if tonumber(self.db.realm.rank) > tonumber(self.db.realm.offlinePlayers[main].rank) then
			--self:Print('Kicked '.. playerName)
			self:SendKick(playerName)
		--else
		--	self:Print('You do not have the rank to kick '.. playerName)
		--end
	end
end

function MG:InitPopupDialogs()
	StaticPopupDialogs["REMOVE_METAGUILDMEMBER"] = {
		text = format(REMOVE_GUILDMEMBER_LABEL, "XXX"),
		button1 = YES,
		button2 = NO,
		OnAccept = function(self)
			local name, main = GetMetaGuildRosterSelection()
			MetaGuildKick(name)
			getglobal("MetaGuild_PlayerInfoFrame"):Hide()
		end,
		OnShow = function(self)
			local name, main = GetMetaGuildRosterSelection()
			self.text:SetFormattedText(L["PlayerInfoFrame/RemoveWarning"], main);
			--format(L["Main/MembersOnlineCount"], i, j)
		end,
		timeout = 0,
		exclusive = 1,
		whileDead = 1,
		hideOnEscape = 1
	};
	StaticPopupDialogs["ADD_METAGUILDMEMBER"] = {
		text = L["Info/Add_Member_Title"],
		button1 = L["Accept"],
		button2 = L["Cancel"],
		hasEditBox = 1,
		--autoCompleteParams = AUTOCOMPLETE_LIST.GUILD_INVITE,
		maxLetters = 12,
		OnAccept = function(self)
			MetaGuildInvite(self.editBox:GetText());
			
		end,
		OnShow = function(self)
			self.editBox:SetFocus();
		end,
		OnHide = function(self)
			ChatEdit_FocusActiveWindow();
			self.editBox:SetText("");
		end,
		EditBoxOnEnterPressed = function(self)
			local parent = self:GetParent();
			MetaGuildInvite(parent.editBox:GetText());
			parent:Hide();
		end,
		EditBoxOnEscapePressed = function(self)
			self:GetParent():Hide();
		end,
		timeout = 0,
		exclusive = 1,
		whileDead = 1,
		hideOnEscape = 1
	}
end

function MG:IsPlayersCharacter(name)
	local main = self:GetMainFromName(UnitName("player"))
	if main ~= nil and self.db.realm.offlinePlayers[main] ~= nil then
		for k, v in pairs(self.db.realm.offlinePlayers[main].characters) do
			if k == name then
				return true
			end
		end
	end
	return false
end


function MG:RemoveMainFromOnlineList(main)
	for k, v in pairs(self.db.realm.onlinePlayers) do
		if v.main == main then
			self.db.realm.onlinePlayers[k] = nil
			return
		end
	end
end

function MG:FirstToUpper(str)
    return (str:gsub("^%l", string.upper))
end


function MG:GetOnlineNameFromMain(main)
	for k, v in pairs(self.db.realm.onlinePlayers) do
		self:Print("A: ".. strlower(main)..", B: "..strlower(v.main))
		if strlower(v.main) == strlower(main) then
			return k
		end
	end
	self:Print("Could not find character for "..main)
	return nil
end

function MG:GetMainFromName(name)
	local main
	name = self:FirstToUpper(name)
	if self.db.realm.onlinePlayers[name] ~= nil then
		--The player is online, this should be easy.
		return self.db.realm.onlinePlayers[name].main
	else
		--The player is offline, this will be a bit more work.
		for a, b in pairs(self.db.realm.offlinePlayers) do
			for c, d in pairs(b.characters) do
				if c == name then
					return a
				end
			end
		end
	end
	return nil
end

function MG:GetRank(name)
	local main = self:GetMainFromName(name)
	if main ~= nil then 
		return tonumber(self.db.realm.offlinePlayers[main].rank)
	end
	return
end

function MetaGuildGetCharacterInfo(name)
	return self:GetCharacterInfo(name)
end


function MetaGuildCreateGuild()
	self:Print("Creating MetaGuild")
	
	
	PlaySound("LEVELUPSOUND")
	
end

function MetaGuildRosterSetPublicNote(name, author, note)
	if name ~= nil then
		self:SendNote("PUBLIC", author, name, note)
	else
		self:Print("Name was not valid")
	end
end

function MetaGuildRosterSetOfficerNote(name, author, t)
	if main ~= nil and name ~= nil then
		self.db.realm.offlinePlayers[main].characters[name].officer.text = t
		self.db.realm.offlinePlayers[main].characters[name].officer.author = author
		self.db.realm.offlinePlayers[main].characters[name].officer.time = timeSent
		MetaGuild_PlayerinfoUpdate(name, main)
	else
		self:Print("Name was not valid")
	end
end

function GetMetaGuildRosterSelection()
	local RosterFrame = getglobal("MetaGuildRosterFrame")
	return RosterFrame.selectedGuildMember.name, RosterFrame.selectedGuildMember.main
end

function MG:ConfirmGuildInvite(sender, guildName, guildChannel, guildPassword, timeOffset)
	StaticPopupDialogs["ACCEPT_METAGUILD_INVITE"] = {
		text =  L["Info/AcceptGuildInviteText"],
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			self.db.realm.timeOffset = timeOffset
			--self:SetTimeCodeOffset(timeOffset)
			self.db.realm.guildName = guildName
			self.db.realm.guildChannel = guildChannel
			self.db.realm.guildPassword = guildPassword
			self.db.realm.offlinePlayers = {}
			self.db.realm.onlinePlayers = {}
			self.db.realm.main = UnitName("player")
			self.db.realm.joined = self:GetTimeCode()
			self:JoinMetaGuildChannels()
			
			self:Print("Setting Main to "..self.db.realm.main)
			if self.db.realm.GuildChannel then 
				self:RegisterComm(string.upper(self.db.realm.GuildChannel))
			end
			self:ScheduleTimer(MetaGuildSendJoin, 1)
			self:AddNewsEvent(player, self.db.realm.joined, 3, 0)

		end,
		sound = "levelup2",
		timeout = 30,
		whileDead = true,
		hideOnEscape = true,
	}

	StaticPopup_Show ("ACCEPT_METAGUILD_INVITE", sender, guildName)
end


function MG:DoRemove(main, kicker, time)
	if main ~= nil then
		if main == self.db.realm.main then
			--You are to remove yourself
			self:Print("You have been removed from "..self.db.realm.guildName)
			if self.db.realm.GuildChannel then 
				LeaveChannelByName(self.db.realm.guildChannel);
			end
			self.db.realm.guildName = ""
			self.db.realm.guildChannel = ""
			self.db.realm.guildPassword = ""
			self.db.realm.timeOffset = 0
			self.db.realm.onlinePlayers = {}
			self.db.realm.offlinePlayers = {}
			self.db.realm.mission = {["time"]=0}
			self.db.realm.motd = {["time"]=0}
			self.db.realm.info = {["time"]=0}
		else
			self:Print(main.." was removed from "..self.db.realm.guildName)
			self:RemoveMainFromOnlineList(main)
			self.db.realm.kickedPlayers[main] = {["time"]=time, ["kicker"]=kicker, ["characters"] = self.db.realm.offlinePlayers[main].characters}
			self.db.realm.offlinePlayers[main] = nil
		end
		self:UpdateOnlineCount()
	end
end

function MG:DoPromote(sendTime)
	self.db.realm.rank =  math.min(3, self.db.realm.rank + 1)
	self:AddNewsEvent(player, sendTime, 3, self.db.realm.rank)
	self:SendUpdate()
end

function MG:DoDemote(sendtime)
	self.db.realm.rank =  math.max(0, self.db.realm.rank - 1)
	self:AddNewsEvent(player, sendTime, 3, (self.db.realm.rank*-1))
	self:SendUpdate()
end

function MG:SlashCommand(input)
	input = strlower(input)
	cmd, arg = strsplit(" ", input)
	if not input or input:trim() == "" or cmd == "usage" then
		--InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
		--LibStub("AceConfigCmd-3.0").HandleCommand(MG, "mg", "MetaGuild", input)
		MetaGuild_Toggle()
		return
	elseif cmd == "test" then
			self:Test(arg)
			--self:SendTest(arg)
		return
	elseif cmd == "invite" then
		if arg and arg:trim() ~= "" then
			MetaGuildInvite(arg)
		end
		return
	elseif cmd == "kick" then
		if arg and arg:trim() ~= "" then
			MetaGuildKick(arg)
		end
		return
	elseif cmd == "offline" then
		self:DisplayPlayerList(self.db.realm.offlinePlayers, true)
		return
	elseif cmd == "online" then
		self:DisplayPlayerList(self.db.realm.onlinePlayers, false)
		return
	elseif cmd == "sync" then
		if arg ~= nil then
			if arg == "on" then
				doingSync = true
				self:Print("Sync On")
			elseif arg == "off" then
				doingSync = false
				self:Print("Sync Off")
			elseif arg == "state" then
				doingSync = false
				if doingSync then
					self:Print("Sync On")
				else
					self:Print("Sync Off")
				end
			else
				self:SyncDatabaseWithPlayer(arg)
			end
		else
			self:StartSyncDatabase()
		end
		return
	elseif cmd == "repair" then
		self:RepairDatabase(arg)
		return
	elseif cmd == "time" then
		if arg == "send" then
			self:SendTimeCode()
		elseif arg == "get" then
			self:RequestTimeCode()
		elseif arg == "offset" then
			self:Print("TimeOffset: "..self.db.realm.timeOffset)			
		else
			self:Print("TimeCode: "..self:GetTimeCode())
		end
		return

	elseif cmd == "version" then
		self:SendVersionRequest("1")	
		return
		
	elseif cmd == "query" then
		if arg == "sync" then
			if doingSync then
				self:Print("Sync: Yes")
			else
				self:Print("Sync: No")
			end
		end
		return		
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(MG, "mg", "MetaGuild", input)
	end
end

function MG:tableMerge(t1, t2)
	--> Merge table2 into table1. Table 2's data will overwrite table 1 <-- 
	for k,v in pairs(t2) do
		if type(v) == "table" then
			if type(t1[k] or false) == "table" then
				tableMerge(t1[k] or {}, t2[k] or {})
			else
				t1[k] = v
			end
		else
			t1[k] = v
		end
	end
	return t1
end


-- Rank System
function MG:GetRankName(index)
	if type(index) == "number" and index >= 0 and index <= 3 then
		local rankName = {[0] = L["Recruit"], [1] = L["Member"], [2] = L["Officer"], [3] = L["GuildMaster"]}
		return rankName[index]
	else
		if type(index) ~= "number" then
			self:Print("Index is not a number: "..type(index))
		end
		self:Print("Index is wrong: "..index)
		return "ERROR"
	end
end

--Status System
function MG:GetStatusText(status)
	if status ~= nil then
		local statusText = {[0]="Offline", [1]="Online", [2]="AFK", [3]="DND", [4]="Unknown"}
		return statusText[status]
	else
		return "Unknown"
	end
end

--Message Template
function MG:CreateMsgTemplate(dataFieldCount)
	return string.rep(specialChar .. "%s", dataFieldCount) .. specialChar;
end

function MG:MetaGuildTextEditFrame_Hide()
	if MetaGuildTextEditFrame:IsShown() then
		MetaGuildTextEditFrame:Hide()
	end
end

function MetaGuildTextEditFrame_Show(editType)
	local frame = getglobal("MetaGuildTextEditFrame")

	if ( editType == "motd" ) then
		MetaGuildTextEditFrame:SetHeight(162)
		MetaGuildTextEditBox:SetMaxLetters(128)
		--MetaGuildTextEditBox:SetText(GetGuildRosterMOTD())
		MetaGuildTextEditFrameTitle:SetText(L["Info/MotD"])
	elseif ( editType == "info" ) then
		MetaGuildTextEditFrame:SetHeight(295)
		MetaGuildTextEditBox:SetMaxLetters(500)
		--MetaGuildTextEditBox:SetText(GuildInfoFrame.cachedInfoText)
		MetaGuildTextEditFrameTitle:SetText(L["Info/Info"])
	elseif ( editType == "mission" ) then
		MetaGuildTextEditFrame:SetHeight(295)
		MetaGuildTextEditBox:SetMaxLetters(500)
		--MetaGuildTextEditBox:SetText(self.db.realm.mission.text)
		MetaGuildTextEditFrameTitle:SetText(L["Guild/Mission"])
	end
	--MetaGuildTextEditBox:SetScript("OnEnterPressed", MetaGuildTextEditFrame_OnAccept);
	MetaGuildTextEditFrame.type = editType;
	if self.db.realm[editType].text == nil then
		self.db.realm[editType].text = ""
	end
	
	--Set the text in the box
	MetaGuildTextEditBox:SetText(self.db.realm[editType].text);
	
	if MetaGuildTextEditFrame:IsShown() then
		MetaGuildTextEditFrame:Hide()
	else
		MetaGuildTextEditFrame:Show()
	end
	
	--FUBAR - add in author and date.
	
	MetaGuildTextEditBox:SetCursorPosition(0);
	MetaGuildTextEditBox:SetFocus();
	PlaySound("igMainMenuOptionCheckBoxOn");
end

function MetaGuildTextEditFrame_OnAccept()
	local infoText = MetaGuildTextEditBox:GetText()
	self:SetGuildInfoText(infoText, MetaGuildTextEditFrame.type)
	MetaGuildTextEditFrame:Hide()
end

function MG:SetGuildInfoText(infoText, editType)
	if self.db.realm[editType] == nil then
		self.db.realm[editType] = {["time"] = 0}
	end
	
	self.db.realm[editType].text = infoText
	self.db.realm[editType].time = self:GetTimeCode()
	self.db.realm[editType].author = self.db.realm.main
	if editType == "mission" then
		MetaGuildGuild_Update()
		--Init a datasync for the mission only.
	elseif editType == "motd" or editType == "info" then
		MetaGuildInfo_Update()
	end
end

function MG:PackData(data)
	--Serialize and compress the data
	local serialData = self:Serialize(data)
	local compressData = LibC:CompressHuffman(serialData)
	return LibCE:Encode(compressData)
end

function MG:UnpackData(encodedData)
	-- Decode the compressed data
	local decodedData = LibCE:Decode(encodedData)
	
	--Decompress the decoded data
	local decompressedData, message = LibC:Decompress(decodedData)
	if(not decompressedData) then
		self:Print("Error decompressing: " .. message)
		return
	end
	
	-- Deserialize the decompressed data
	local success, data = self:Deserialize(decompressedData)
	if (not success) then
		print("Error deserializing " .. data)
		return
	end
	return data
end

function MG:DataTest()
	local data = {["msg"]="Druids Rock", ["time"]="20203920"}
	local packed = self:PackData(data)

	self:Print("Packed Data: "..packed)

	local unpacked = self:UnpackData(packed)
	self:Print("Data Test: "..unpacked.msg)
end

--Time Functions

function MG:GetTimeCode(t)
	if t == nil then
		t = time()
	end
	t = tonumber(t)
	return (t + self.db.realm.timeOffset)
end

function MG:GetTimeCodeOffset(t)
	if t ~= nil then
		if self.db.realm.rank == 3 then
			return 0
		else
			return tonumber(t) - time()
		end
	else
		return self.db.realm.timeOffset
	end
end

function MG:TimeSinceLastTimeCode(lastTime)
	local nowDate = date("*t", self:GetTimeCode())
	local lastDate = date("*t", lastTime)
	
	--self:DisplayDateTable(nowDate)
	--self:DisplayDateTable(lastDate)
	
	return self:TimeDifference(nowDate, lastDate)
end


function MG:GetServerSeconds()
	local hour, minute = GetGameTime()
	if self.lastMinute ~= minute then
		if self.debug then
			self:Print("Updating minute to "..minute)
		end
		local _
		_,self.lastMinute = GetGameTime()
		self.secondsDifference = mod(time(), 60)
	elseif self.secondsDifference == nil then
		self.secondsDifference = 0
	end
	return format("%02d", mod(time() - self.secondsDifference, 60))
end

function MG:GetCurrentTimeCode()

	--IMPORTANT: due to the way we are getting the seconds off the server time, 
	--we must wait a minute before actually calling GetCurrentTimeCode() for best accuracy

	local serverOffset = self:GetServerOffset()
	local hour, minute = GetGameTime()
	local year, month, day, localHour, second = date("%Y"), date("%m"), date("%d"), date("%H"), date("%S")
	
	self:Print("Server Offset: "..serverOffset)
	self:Print("GHour: "..hour)
	self:Print("LHour: "..localHour)
	
	second = self:GetServerSeconds()

	if (localHour - serverOffset < 0) then
		--Re-adjust the date to refect that the server is still living out yesterday.
		--do we need to flip back a year?
		if month == 1 and day == 1 then
			year = year - 1
		end
		--do we need to flip back a month?
		if day == 1 then
			if month == 1 then
				month = 12
			else
				month = month - 1
			end
			day = self:GetDaysInMonth(month, year)
		end
	end
	
	local dateTable = {["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour, ["min"] = minute, ["sec"] = second}
	--date = {["year"] = 1969, ["month"] = 1, ["day"] = 1, ["hour"] = 0, ["min"] = 0, ["sec"] = 48}
	
	
	
	return time(dateTable)
	--return time(date)
end

function MG:PadNumber(number, padding)
	return string.format("%0"..padding.."d", number)
end

function MG:Date2TimeCode(date)
	--Must Pad the numbers to keep the format
	date.year = self:PadNumber(date.year, 4)
	date.month = self:PadNumber(date.month, 2)
	date.day = self:PadNumber(date.day, 2)
	date.hour = self:PadNumber(date.hour, 2)
	date.min = self:PadNumber(date.min, 2)
	date.sec = self:PadNumber(date.sec, 2)
	return (date.year..date.month..date.day..date.hour..date.min..date.sec)
end

function MG:TimeCode2String(timeCode)
	--self:Print("Tyep: "..type(timeCode))
	date = self:TimeCode2Date(timeCode)
	--self:Print("Date: "..self:Date2String(date))
	return date("%B %d, %Y %X",  time(date))
end






function MG:Date2String(date)
	return (date.year..", "..date.month..", "..date.day..", "..date.hour..", "..date.min..", "..date.sec)
end

function MG:TimeSinceTimeCode(oldTimeCode)
	local currentTimeCode = self:GetCurrentTimeCode()
	local currentDate = self:TimeCode2Date(currentTimeCode)
	local oldDate = self:TimeCode2Date(oldTimeCode)
	local diffDate = self:TimeDifference(currentDate, oldDate)
	--self:Print("FinalDiffDate: "..self:Date2String(diffDate))
	return diffDate
end

function MG:TimeCode2Date(timeCode)
	--self:Print("TimeCode: "..timeCode)
	local year = tonumber(string.sub(timeCode, 0, 4))
	local month = tonumber(string.sub(timeCode, 5, 6))
	local day = tonumber(string.sub(timeCode, 7, 8))
	local hour = tonumber(string.sub(timeCode, 9, 10))
	local minute = tonumber(string.sub(timeCode, 11, 12))
	local second = tonumber(string.sub(timeCode, 13, 14))
	return {["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour, ["min"] = minute, ["sec"] = second}
end




function MG:GetDaysInMonth(month, year)
	local feb = 28
	if self:isLeapYear(year) then
		feb = 29
	end
	local daysInMonths = {31, feb, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
	return daysInMonths[month]
end

function MG:IsLeapYear(year)
--A year will be a leap year if it is divisible by 4 but not by 100. 
--If a year is divisible by 4 and by 100, it is not a leap year unless it is also divisible by 400. 
	if ((year % 4 == 0 and year % 100 ~= 0) or year % 400 == 0) then
		return 1
	else
		return 0
	end
end


function MG:GetLocalOffset()
	if self.localOffset ~= nil then
		return self.localOffset
	end
	local localHour = tonumber(date("%H"))
	local localMinute = tonumber(date("%M"))
	local utcHour = tonumber(date("!%H"))
	local utcMinute = tonumber(date("!%M"))
	local loc = localHour + localMinute / 60
	local utc = utcHour + utcMinute / 60
	self.localOffset = floor((loc - utc) * 2 + 0.5) / 2
	if self.localOffset >= 12 then
		self.localOffset = self.localOffset - 24
	end
	return self.localOffset
end

function MG:GetServerOffset()
	if self.serverOffset ~= nil then
		return self.serverOffset
	end
	local serverHour, serverMinute = GetGameTime()
	local utcHour = tonumber(date("!%H"))
	local utcMinute = tonumber(date("!%M"))
	local ser = serverHour + serverMinute / 60
	local utc = utcHour + utcMinute / 60
	self.serverOffset = floor((ser - utc) * 2 + 0.5) / 2
	if self.serverOffset >= 12 then
		self.serverOffset = self.serverOffset - 24
	elseif self.serverOffset < -12 then
		self.serverOffset = self.serverOffset + 24
	end
	return self.serverOffset
end

function MG:TimeDifference(dateA, dateB)
	local dateC = {}
	dateC.year = dateA.year - dateB.year
	dateC.month = dateA.month - dateB.month
	dateC.day = dateA.day - dateB.day
	dateC.hour = dateA.hour - dateB.hour
	dateC.min = dateA.min - dateB.min
	dateC.sec = dateA.sec - dateB.sec
	return self:ConformDate(dateC)
end

function MG:TimeProduct(dateA, dateB)
	local dateC = {}
	dateC.year = dateA.year + dateB.year
	dateC.month = dateA.month + dateB.month
	dateC.day = dateA.day + dateB.day
	dateC.hour = dateA.hour + dateB.hour
	dateC.min = dateA.min + dateB.min  
	dateC.sec = dateA.sec + dateB.sec	
	return self:ConformDate(dateC)
end

function MG:ConformDate(date)
	--self:Print("ConformDate: "..self:Date2String(date))
		
	--self:DisplayDateTable(date)
		
	local daysInYear = {[0] = 365, [1] = 366}
	local daysInMonth = {[0] = 0, [1] = 31, [2] = 29, [3] = 31, [4] = 30, [5] = 31, [6] = 30, [7] = 31, [8] = 31, [9] = 30, [10] = 31, [11] = 30, [12] = 31}
	--Seconds
	date.min = date.min + floor(date.sec / 60.0)
	date.sec = floor(date.sec % 60.0)
	if date.sec < 0 then
		date.sec = date.sec + 60
		date.min = date.min - 1
	end
	
	--Minutes
	date.hour = date.hour + floor(date.min / 60.0)
	date.min = floor(date.min % 60.0)
	if date.min < 0 then
		date.min = date.min + 60
		date.hour = date.hour - 1
	end
	
	--Hours
	date.day = date.day + floor(date.hour / 24.0)
	date.hour = floor(date.hour % 24.0)
	if date.hour < 0 then
		date.hour = date.hour + 24
		date.day = date.day - 1
	end
	
	--Months Phase 1
	date.year = date.year + floor(date.month / 12.0)
	date.month = floor(date.month % 12.0)
	if date.month < 0 then
		date.month = date.month + 12
		date.year = date.year - 1
	end
	
	--Days Phase 1
	if date.day >= 0 then
		catch = 0
		while (date.day > daysInYear[self:IsLeapYear(date.year)] and catch < 40) do 
			date.day = date.day - daysInYear[self:isLeapYear(date.year)]
			date.year = date.year + 1
			catch = catch + 1
		end
	else
		--Subtracting days
		catch = 0
		while(date.day < (daysInYear[self:IsLeapYear(date.year)]*-1) and catch < 40) do 
			date.day = date.day + daysInYear[self:isLeapYear(date.year)]
			date.year = date.year - 1
			catch = catch + 1
		end
	end
	
	--Month Phase 2
	local catch = 0
	if date.day >= 0 then
		catch = 0
--		self:Print("Day: "..date.day)
--		self:Print("Month: "..date.month)
--		self:Print("Days In Month: "..daysInMonth[0])
		while(date.day > daysInMonth[date.month] and catch < 40) do
			date.day = date.day - daysInMonth[date.month]
			date.month = date.month + 1
			if date.month > 12 then
				date.month = 12
				date.year = date.year - 1
			end
			catch = catch + 1
		end
	else
		--Subtracting days
		catch = 0
		while(date.day < (daysInYear[self:IsLeapYear(date.year)] * -1) and catch < 40) do 
			date.day = date.day + daysInYear[self:IsLeapYear(date.year)]
			date.month = date.month - 1
			if date.month < 1 then 
				date.month = 12
				date.year = date.year - 1
			end
			catch = catch + 1
		end
	end
	--self:Print("FinalDate: "..self:Date2String(date))
	return date
end

function MG:GetLastTimeOnline(t, status)
	local lastOnDate = self:TimeSinceLastTimeCode(t)
	return self:displayTimeSinceLastOn(lastOnDate, status)
end

function MG:DisplayDateTable(d)
	self:Print(d.year.."-"..d.month.."-"..d.day.."   "..d.hour..":"..d.min..":"..d.sec)
end


function MG:displayTimeSinceLastOn(d, status)
	--d = date("*t", timeCode)
	
	
	--self:DisplayDateTable(d)	
	
	
	local a = "Seconds"
	if status == 1 or status == 2 or status == 3 then
		return self:GetStatusText(status)
	else
		if d.sec == 1 then
			a = d.sec.." "..L["Time/Second"]
		end

		if d.sec > 1 then
			a = d.sec.." "..L["Time/Seconds"]
		end

		if d.min == 1 then
			a = d.min.." "..L["Time/Minute"]
		end

		if d.min > 1 then
			a = d.min.." "..L["Time/Minutes"]
		end

		if d.hour == 1 then
			a = d.hour.." "..L["Time/Hour"]
		end

		if d.hour > 1 then
			a = d.hour.." "..L["Time/Hours"]
		end

		if d.day == 1 then
			a = d.day.." "..L["Time/Day"]
		end

		if d.day > 1 then
			a = d.day.." "..L["Time/Days"]
		end

		if d.month == 1 then
			a = d.month.." "..L["Time/Month"]
		end

		if d.month > 1 then
			a = d.month.." "..L["Time/Months"]
		end

		if d.year == 1 then
			a = d.year.." "..L["Time/Year"]
		end

		if d.year > 1 then
			a = d.year.." "..L["Time/Years"]
		end
		
		return a
	end
end

function MG:Test(index)	
--	self:Print(date("%B %d, %Y %X",  self:GetCurrentTimeCode()))
--	self:Print(date("%B %d, %Y %X",  self:GetTimeCode()))
--	local personalBGRating, ratedRbgReward, allRbgPointsThisWeek,allRbgMaxPointsThisWeek,rbgPointsThisWeek,rbgMaxPointsThisWeek, rbgSeasonWins, rbgSeasonPlayed, rbgWeeklyWins, rbgWeeklyPlayed = GetPersonalRatedBGInfo()
	
--	local personalArenaRating2v2, ratedArenaReward2v2, allPointsThisWeek2v2, allMaxPointsThisWeek2v2,arenaPointsThisWeek2v2,arenaMaxPointsThisWeek2v2 = GetPersonalRatedArenaInfo(1)
	
--	if index == nil then
--		index = 1
--	end


	--Time to Get the Arena info
--	local arenaInfo = {}
--	for index=1, 3 do	
--		ArenaTeamRoster(index)
--		local teamName, teamSize, teamRating, weekPlayed, weekWins, seasonPlayed, seasonWins, playerPlayed, seasonPlayerPlayed, teamRank, playerRating = GetArenaTeam(index)
--		if teamSize ~= nil then
--			arenaInfo[teamSize] = {["name"]=teamName, ["rating"]=teamRating, ["playerRating"] = playerRating, ["teamRank"]=teamRank}
--		end
--	end

	
--	if arenaInfo[2] ~= nil then
--		self:Print("2v2 Team Name: "..arenaInfo[2].name)
--		self:Print("2v2 Team Rating: "..arenaInfo[2].rating)
--		self:Print("2v2 Team Rank: "..arenaInfo[2].teamRank)
--		self:Print(" ")
--	end
	
--	if arenaInfo[3] ~= nil then
--		self:Print("3v3 Team Name: "..arenaInfo[3].name)
--		self:Print("3v3 Team Rating: "..arenaInfo[3].rating)
--		self:Print("3v3 Team Rank: "..arenaInfo[3].teamRank)
--		self:Print(" ")
--	end
	
--	if arenaInfo[5] ~= nil then
--		self:Print("5v5 Team Name: "..arenaInfo[5].name)
--		self:Print("5v5 Team Rating: "..arenaInfo[5].rating)
--		self:Print("5v5 Team Rank: "..arenaInfo[5].teamRank)
--		self:Print(" ")
--	end


	
--	self:Print(" ")
--	self:Print("****Rated Battle Grounds**** ")
--	self:Print("RBG Rating: "..personalBGRating)
--	self:Print("Rated BG Reward: "..ratedRbgReward)
--	self:Print("All Points This Week: "..allRbgPointsThisWeek)
--	self:Print("All Max Points This Week: "..allRbgMaxPointsThisWeek)
--	self:Print("RBG Points This Week: "..rbgPointsThisWeek)
--	self:Print("RBG Max Points This Week: "..rbgMaxPointsThisWeek)
--	self:Print("Season Wins?: "..rbgSeasonWins)
--	self:Print("Seasons Played?: "..rbgSeasonPlayed)
--	self:Print("Weekly Wins: "..rbgWeeklyWins)
--	self:Print("Weekly Played: "..rbgWeeklyPlayed)
--	self:Print(" ")
--	self:Print("****Rated 2v2**** ")
--	self:Print("Arena Rating 2v2: "..personalArenaRating2v2)
--	self:Print("Arena Reward 2v2: "..ratedArenaReward2v2)
--	self:Print("All Points This Week: "..allPointsThisWeek2v2)
--	self:Print("All Max Points This Week: "..allMaxPointsThisWeek2v2)
--	self:Print("Arena 2v2 Points This Week: "..arenaPointsThisWeek2v2)
--	self:Print("Arena 2v2 Max Points This Week: "..arenaMaxPointsThisWeek2v2)

	--self.db.realm.guildChannel = self:GetRandomString(10)
	--self.db.realm.guildPassword = self:GetRandomString(10)


--	local list = GetCategoryList()
--	for k,CategoryID in pairs(list) do
		
--		local Title, ParentCategoryID, Something = GetCategoryInfo(CategoryID)
--		ParentCategoryID GetCategoryInfo(CategoryID)
--		self:Print(CategoryID..":"..Title)
--	end
	
	
	--self:SendAchievement(2336)
	
	--self:SendLevelUp(85)
	
	--MG:AddNewsEvent(name, sendTime, eventTypeId, data)
	
	
	id = 2336
	--id = 4989
	if id ~= nil and not select(12,GetAchievementInfo(id)) then
		self:Print("got it")
		self:SendAchievement(id)
	end
	
end

--Random String

function MG:GetRandomString(length)
	-- Length (number)

	local chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
	--local chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
	--local chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	local result = ""
	local range = strlen(chars)
	for loop = 1,length do
		local charNum = math.random(1, range)
		result = (result..string.sub(chars, charNum, charNum))
	end
	return result
end


